@@ -7,10 +7,9 @@ optionally for plotting support.
 
 See the webpage for more information and documentation:
 
-    http://code.google.com/p/sympy/
-"""
+    http://code.google.com/p/sympy/"""
 
-__version__ = "0.6.4"
+__version__ = "0.6.6"
 
 
 def __sympy_debug():
@@ -21,10 +20,11 @@ SYMPY_DEBUG = __sympy_debug()
 
 import symbol as stdlib_symbol
 from sympy.core import *
-
+from assumptions import *
 from polys import *
 from series import *
 from functions import *
+from logic import *
 from ntheory import *
 from concrete import *
 from simplify import *
@@ -38,7 +38,7 @@ from integrals import *
 from plotting import Plot, textplot
 from printing import pretty, pretty_print, pprint, pprint_use_unicode, \
     pprint_try_use_unicode, print_gtk, print_tree
-from printing import ccode, latex, preview, view, pngview, pdfview, dviview
+from printing import ccode, latex, preview
 from printing import python, print_python, srepr, sstr, sstrrepr
 
 evalf._create_evalf_table()
