@@ -1,6 +1,7 @@
 from sympy import (Symbol, Sum, oo, Real, Rational, sum, pi, cos, zeta,
-    Catalan, log, factorial, sqrt, E, sympify, binomial, EulerGamma, Function)
+Catalan, exp, log, factorial, sqrt, E, sympify, binomial, EulerGamma, Function, Integral, Product, product)
 from sympy.concrete.summations import getab
+from sympy.concrete.sums_products import Sum2
 from sympy.utilities.pytest import XFAIL
 
 a, b, c, d, m, k = map(Symbol, 'abcdmk')
@@ -164,3 +165,27 @@ def test_telescopic_sums():
     assert Sum(f(k)-f(k+2),(k,m,n)).doit() == -f(1+n) - f(2+n) + f(m) + f(1+m)
     assert Sum(cos(k)-cos(k+3),(k,1,n)).doit() == -cos(1 + n) - cos(2 + n) - \
                                            cos(3 + n) + cos(1) + cos(2) + cos(3)
+
+def test_sum_reconstruct():
+    s = Sum(n**2, (n, -1, 1))
+    assert s == Sum(*s.args)
+
+def test_Sum_limit_subs():
+    assert Sum(a*exp(a), (a, -2, 2)) == Sum(a*exp(a), (a, -b, b)).subs(b,2)
+
+def test_Sum2():
+    x = Symbol('x')
+    y = Symbol('y')
+    assert Sum2(x**y, (x, 1, 3)) == 1 + 2**y + 3**y
+
+def test_Sum_doit():
+    assert Sum(n*Integral(a**2), (n, 0, 2)).doit() == a**3
+    assert Sum(n*Integral(a**2), (n, 0, 2)).doit(deep = False) == \
+        3*Integral(a**2)
+    assert sum(n*Integral(a**2), (n, 0, 2)) == 3*Integral(a**2)
+
+def test_Product_doit():
+    assert Product(n*Integral(a**2), (n, 1, 3)).doit() == 2 * a**9 / 9
+    assert Product(n*Integral(a**2), (n, 1, 3)).doit(deep = False) == \
+        6*Integral(a**2)**3
+    assert product(n*Integral(a**2), (n, 1, 3)) == 6*Integral(a**2)**3
