@@ -4,18 +4,17 @@ for mathematical functions.
 """
 
 from sympy.mpmath.libmpf import (from_int, from_rational, fzero, normalize,
-        bitcount, round_nearest, to_str, fone, fnone, fhalf, to_float,
-        from_float, fnone, to_int, mpf_lt, mpf_sqrt, mpf_cmp, mpf_abs,
+        bitcount, round_nearest, to_str, fone, fnone, fhalf, from_float,
+        to_float, fnone, to_int, mpf_lt, mpf_sqrt, mpf_cmp, mpf_abs,
         mpf_pow_int, mpf_shift, mpf_add, mpf_mul, mpf_neg)
 
 import sympy.mpmath.libmpc as libmpc
 from sympy.mpmath.settings import dps_to_prec
 from sympy.mpmath import mpf, mpc, quadts, quadosc, mp, make_mpf
-from sympy.mpmath.gammazeta import mpf_gamma
 from sympy.mpmath.libelefun import mpf_pi, mpf_log, mpf_pow, mpf_sin, mpf_cos, \
         mpf_atan, mpf_atan2, mpf_e, mpf_exp
 from sympy.mpmath.libmpf import MP_BASE, from_man_exp
-from sympy.mpmath.calculus import shanks, richardson, nsum
+from sympy.mpmath.calculus import nsum
 from sympy.mpmath import inf as mpmath_inf
 
 from sympy.mpmath.gammazeta import mpf_bernoulli
@@ -23,7 +22,6 @@ from sympy.mpmath.gammazeta import mpf_bernoulli
 import math
 
 from basic import Basic, C, S
-from function import Function
 from sympify import sympify
 
 LG10 = math.log(10,2)
@@ -48,7 +46,13 @@ class PrecisionExhausted(ArithmeticError):
 
 """
 An mpf value tuple is a tuple of integers (sign, man, exp, bc)
-representing a floating-point numbers.
+representing a floating-point number: (-1)**sign*man*2**exp where
+bc should correspond to the number of bits used to represent the
+mantissa (man) in binary notation, e.g. (0,5,1,3) represents 10::
+
+>>> from sympy.core.evalf import bitcount
+>>> n=(-1)**0 * 5 * 2**1; n, bitcount(5)
+(10, 3)
 
 A temporary result is a tuple (re, im, re_acc, im_acc) where
 re and im are nonzero mpf value tuples representing approximate
@@ -61,12 +65,31 @@ if the corresponding complex part is None.
 """
 
 def fastlog(x):
-    """Fast approximation of log2(x) for an mpf value tuple x."""
+    """Fast approximation of log2(x) for an mpf value tuple x.
+
+    Notes: Calculated as exponent + width of mantissa. This is an
+    approximation for two reasons: 1) it gives the ceil(log2(abs(x)))
+    value and 2) it is too high by 1 in the case that x is an exact
+    power of 2. Although this is easy to remedy by testing to see if
+    the odd mpf mantissa is 1 (indicating that one was dealing with
+    an exact power of 2) that would decrease the speed and is not
+    necessary as this is only being used as an approximation for the
+    number of bits in x. The correct return value could be written as
+    "x[2] + (x[3] if x[1] != 1 else 0)".
+        Since mpf tuples always have an odd mantissa, no check is done
+    to see if the mantissa is a multiple of 2 (in which case the
+    result would be too large by 1).
+
+    Example::
+
+    >>> from sympy import log
+    >>> from sympy.core.evalf import fastlog, bitcount
+    >>> n=(-1)**0*5*2**1; n, (log(n)/log(2)).evalf(), fastlog((0,5,1,bitcount(5)))
+    (10, 3.32192809488736, 4)
+    """
+
     if not x or x == fzero:
         return MINUS_INF
-    # log2(x) ~= exponent + width of mantissa
-    # Note: this actually gives ceil(log2(x)), which is a useful
-    # feature for interval arithmetic.
     return x[2] + x[3]
 
 def complex_accuracy(result):
@@ -245,6 +268,7 @@ def add_terms(terms, prec, target_prec):
             # XXX: this is supposed to represent a scaled zero
             return mpf_shift(fone, target_prec), -1
         return terms[0]
+    max_extra_prec = 2*prec
     sum_man, sum_exp, absolute_error = 0, 0, MINUS_INF
     for x, accuracy in terms:
         if not x:
@@ -255,16 +279,22 @@ def add_terms(terms, prec, target_prec):
         absolute_error = max(absolute_error, bc+exp-accuracy)
         delta = exp - sum_exp
         if exp >= sum_exp:
-            if delta > 4*prec:
+            # x much larger than existing sum?
+            # first: quick test
+            if (delta > max_extra_prec) and \
+                ((not sum_man) or delta-bitcount(abs(sum_man)) > max_extra_prec):
                 sum_man = man
                 sum_exp = exp
             else:
-                sum_man += man << delta
+                sum_man += (man << delta)
         else:
-            if (-delta) > 4*prec:
-                pass
+            delta = -delta
+            # x much smaller than existing sum?
+            if delta-bc > max_extra_prec:
+                if not sum_man:
+                    sum_man, sum_exp = man, exp
             else:
-                sum_man = (sum_man << (-delta)) + man
+                sum_man = (sum_man << delta) + man
                 sum_exp = exp
     if absolute_error == MINUS_INF:
         return None, None
@@ -294,17 +324,17 @@ def evalf_add(v, prec, options):
     try:
         while 1:
             terms = [evalf(arg, prec+10, options) for arg in args]
-            re, re_accuracy = add_terms([(a[0],a[2]) for a in terms if a[0]], prec, target_prec)
-            im, im_accuracy = add_terms([(a[1],a[3]) for a in terms if a[1]], prec, target_prec)
-            accuracy = complex_accuracy((re, im, re_accuracy, im_accuracy))
+            re, re_acc = add_terms([(a[0],a[2]) for a in terms if a[0]], prec, target_prec)
+            im, im_acc = add_terms([(a[1],a[3]) for a in terms if a[1]], prec, target_prec)
+            accuracy = complex_accuracy((re, im, re_acc, im_acc))
             if accuracy >= target_prec:
                 if options.get('verbose'):
-                    print "ADD: wanted", target_prec, "accurate bits, got", re_accuracy, im_accuracy
-                return re, im, re_accuracy, im_accuracy
+                    print "ADD: wanted", target_prec, "accurate bits, got", re_acc, im_acc
+                return re, im, re_acc, im_acc
             else:
                 diff = target_prec - accuracy
                 if (prec-target_prec) > options.get('maxprec', DEFAULT_MAXPREC):
-                    return re, im, re_accuracy, im_accuracy
+                    return re, im, re_acc, im_acc
 
                 prec = prec + max(10+2**i, diff)
                 options['maxprec'] = min(oldmaxprec, 2*prec)
@@ -314,18 +344,6 @@ def evalf_add(v, prec, options):
     finally:
         options['maxprec'] = oldmaxprec
 
-# Helper for complex multiplication
-# XXX: should be able to multiply directly, and use complex_accuracy
-# to obtain the final accuracy
-def cmul((a, aacc), (b, bacc), (c, cacc), (d, dacc), prec, target_prec):
-    A, Aacc = mpf_mul(a,c,prec), min(aacc, cacc)
-    B, Bacc = mpf_mul(mpf_neg(b),d,prec), min(bacc, dacc)
-    C, Cacc = mpf_mul(a,d,prec), min(aacc, dacc)
-    D, Dacc = mpf_mul(b,c,prec), min(bacc, cacc)
-    re, re_accuracy = add_terms([(A, Aacc), (B, Bacc)], prec, target_prec)
-    im, im_accuracy = add_terms([(C, Cacc), (D, Cacc)], prec, target_prec)
-    return re, im, re_accuracy, im_accuracy
-
 def evalf_mul(v, prec, options):
     args = v.args
     # With guard digits, multiplication in the real case does not destroy
@@ -345,16 +363,15 @@ def evalf_mul(v, prec, options):
     # direction tells us that the result should be multiplied by
     # i**direction
     for arg in args:
-        re, im, a, aim = evalf(arg, prec, options)
+        re, im, re_acc, im_acc = evalf(arg, prec, options)
         if re and im:
-            complex_factors.append((re, im, a, aim))
+            complex_factors.append((re, im, re_acc, im_acc))
             continue
         elif re:
-            s, m, e, b = re
+            (s, m, e, b), w_acc = re, re_acc
         elif im:
-            a = aim
+            (s, m, e, b), w_acc = im, im_acc
             direction += 1
-            s, m, e, b = im
         else:
             return None, None, None, None
         direction += 2*s
@@ -364,28 +381,32 @@ def evalf_mul(v, prec, options):
         if bc > 3*prec:
             man >>= prec
             exp += prec
-        acc = min(acc, a)
+        acc = min(acc, w_acc)
     sign = (direction & 2) >> 1
     v = normalize(sign, man, exp, bitcount(man), prec, round_nearest)
     if complex_factors:
-        # Multiply first complex number by the existing real scalar
-        re, im, re_acc, im_acc = complex_factors[0]
-        re = mpf_mul(re, v, prec)
-        im = mpf_mul(im, v, prec)
-        re_acc = min(re_acc, acc)
-        im_acc = min(im_acc, acc)
-        # Multiply consecutive complex factors
-        complex_factors = complex_factors[1:]
+        # make existing real scalar look like an imaginary and
+        # multiply by the remaining complex numbers
+        re, im = v, (0, MP_BASE(0), 0, 0)
         for wre, wim, wre_acc, wim_acc in complex_factors:
-            re, im, re_acc, im_acc = cmul((re, re_acc), (im,im_acc),
-                (wre,wre_acc), (wim,wim_acc), prec, target_prec)
+            # acc is the overall accuracy of the product; we aren't
+            # computing exact accuracies of the product.
+            acc = min(acc,
+                      complex_accuracy((wre, wim, wre_acc, wim_acc)))
+            A = mpf_mul(re, wre, prec)
+            B = mpf_mul(mpf_neg(im), wim, prec)
+            C = mpf_mul(re, wim, prec)
+            D = mpf_mul(im, wre, prec)
+            re, xre_acc = add_terms([(A, acc), (B, acc)], prec, target_prec)
+            im, xim_acc = add_terms([(C, acc), (D, acc)], prec, target_prec)
+
         if options.get('verbose'):
-            print "MUL: obtained accuracy", re_acc, im_acc, "expected", target_prec
+            print "MUL: wanted", target_prec, "accurate bits, got", acc
         # multiply by i
         if direction & 1:
-            return mpf_neg(im), re, re_acc, im_acc
+            return mpf_neg(im), re, acc, acc
         else:
-            return re, im, re_acc, im_acc
+            return re, im, acc, acc
     else:
         # multiply by i
         if direction & 1:
@@ -415,7 +436,7 @@ def evalf_pow(v, prec, options):
             return mpf_pow_int(re, p, target_prec), None, target_prec, None
         # (x*I)**n = I**n * x**n
         if im and not re:
-            z = fpowi(im, p, target_prec)
+            z = mpf_pow_int(im, p, target_prec)
             case = p % 4
             if case == 0: return z, None, target_prec, None
             if case == 1: return None, z, None, target_prec
@@ -513,7 +534,7 @@ def evalf_trig(v, prec, options):
     # 20 extra bits is possibly overkill. It does make the need
     # to restart very unlikely
     xprec = prec + 20
-    re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
+    re, im, re_acc, im_acc = evalf(arg, xprec, options)
     if im:
         raise NotImplementedError
     if not re:
@@ -534,7 +555,7 @@ def evalf_trig(v, prec, options):
     # Very large
     if xsize >= 10:
         xprec = prec + xsize
-        re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
+        re, im, re_acc, im_acc = evalf(arg, xprec, options)
     # Need to repeat in case the argument is very close to a
     # multiple of pi (or pi/2), hitting close to a root
     while 1:
@@ -549,7 +570,7 @@ def evalf_trig(v, prec, options):
             if xprec > options.get('maxprec', DEFAULT_MAXPREC):
                 return y, None, accuracy, None
             xprec += gap
-            re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
+            re, im, re_acc, im_acc = evalf(arg, xprec, options)
             continue
         else:
             return y, None, prec, None
@@ -960,7 +981,7 @@ def evalf(x, prec, options):
     if options.get("verbose"):
         print "### input", x
         print "### output", to_str(r[0] or fzero, 50)
-        #print "### raw", r[0], r[2]
+        print "### raw", r#r[0], r[2]
         print
     if options.get("chop"):
         r = chop_parts(r, prec)
@@ -1022,13 +1043,13 @@ def Basic_evalf(x, n=15, **options):
     re, im, re_acc, im_acc = result
     if re:
         p = max(min(prec, re_acc), 1)
-        #re = fpos(re, p, round_nearest)
+        #re = mpf_pos(re, p, round_nearest)
         re = C.Real._new(re, p)
     else:
         re = S.Zero
     if im:
         p = max(min(prec, im_acc), 1)
-        #im = fpos(im, p, round_nearest)
+        #im = mpf_pos(im, p, round_nearest)
         im = C.Real._new(im, p)
         return re + im*S.ImaginaryUnit
     else:
@@ -1043,8 +1064,8 @@ def N(x, n=15, **options):
     Both .evalf() and N() are equivalent, use the one that you like better.
 
     Example:
-    >>> from sympy import Sum, Symbol, oo
-    >>> k = Symbol("k")
+    >>> from sympy import Sum, Symbol, oo, N
+    >>> from sympy.abc import k
     >>> Sum(1/k**k, (k, 1, oo))
     Sum(k**(-k), (k, 1, oo))
     >>> N(Sum(1/k**k, (k, 1, oo)), 4)
