@@ -4,43 +4,43 @@ There are two types of functions:
    (in the sense that function can be evaluated).
     e = exp
 2) undefined function with a name but no body. Undefined
-  functions can be defined using a Function class as follows:
-    f = Function('f')
-  (the result will be Function instance)
+   functions can be defined using a Function class as follows:
+       f = Function('f')
+   (the result will be a Function instance)
 3) this isn't implemented yet: anonymous function or lambda function that has
-no name but has body with dummy variables. An anonymous function
-   object creation examples:
-    f = Lambda(x, exp(x)*x)
-    f = Lambda(exp(x)*x)  # free symbols in the expression define the number of arguments
-    f = exp * Lambda(x,x)
+   no name but has body with dummy variables. Examples of anonymous function
+   creation:
+       f = Lambda(x, exp(x)*x)
+       f = Lambda(exp(x)*x)  # free symbols in the expression define the number of arguments
+       f = exp * Lambda(x,x)
 4) isn't implemented yet: composition of functions, like (sin+cos)(x), this
-works in sympycore, but needs to be ported back to SymPy.
+   works in sympy core, but needs to be ported back to SymPy.
 
 
 Example:
-    >>> from sympy import *
-    >>> f = Function("f")
-    >>> x = Symbol("x")
+    >>> import sympy
+    >>> f = sympy.Function("f")
+    >>> from sympy.abc import x
     >>> f(x)
     f(x)
-    >>> print srepr(f(x).func)
+    >>> print sympy.srepr(f(x).func)
     Function('f')
     >>> f(x).args
     (x,)
+
 """
 
-from basic import Basic, Atom, S, C, sympify
-from basic import BasicType, BasicMeta
-from operations import AssocOp
+from basic import Basic, Atom, S, C
+from basic import BasicMeta
 from cache import cacheit
 from itertools import repeat
 from numbers import Rational, Integer
 from symbol import Symbol
-from add    import Add
 from multidimensional import vectorize
+from sympy.utilities.decorator import deprecated
+from sympy.utilities import all
 
 from sympy import mpmath
-from sympy.utilities.decorator import deprecated
 
 class PoleError(Exception):
     pass
@@ -125,7 +125,7 @@ class Function(Basic):
             else:
                 print args
                 print type(args[0])
-                raise Exception("You need to specify exactly one string")
+                raise TypeError("You need to specify exactly one string")
 
         # (2) create new instance of a class created in (1)
         #     UC: Function('f')(x)
@@ -138,23 +138,21 @@ class Function(Basic):
         # up to here.
         if options.get('evaluate') is False:
             return Basic.__new__(cls, *args, **options)
-        r = cls.eval(*args)
-        if isinstance(r, Basic):
+        evaluated = cls.eval(*args)
+        if evaluated is not None: return evaluated
+        # Just undefined functions have nargs == None
+        if not cls.nargs and hasattr(cls, 'undefined_Function'):
+            r = Basic.__new__(cls, *args, **options)
+            r.nargs = len(args)
             return r
-        elif r is None:
-            # Just undefined functions have nargs == None
-            if not cls.nargs and hasattr(cls, 'undefined_Function'):
-                r = Basic.__new__(cls, *args, **options)
-                r.nargs = len(args)
-                return r
-            pass
-        elif not isinstance(r, tuple):
-            args = (r,)
         return Basic.__new__(cls, *args, **options)
 
     @property
     def is_commutative(self):
-        return True
+        if all(getattr(t, 'is_commutative') for t in self.args):
+            return True
+        else:
+            return False
 
     @classmethod
     @deprecated
@@ -199,18 +197,12 @@ class Function(Basic):
         elif old.is_Function and new.is_Function:
             if old == self.func:
                 if self.nargs is new.nargs or not new.nargs:
-                    return new(*self.args[:])
+                    return new(*self.args)
                 # Written down as an elif to avoid a super-long line
                 elif isinstance(new.nargs,tuple) and self.nargs in new.nargs:
-                    return new(*self.args[:])
-        obj = self.func._eval_apply_subs(*(self.args[:] + (old,) + (new,)))
-        if obj is not None:
-            return obj
+                    return new(*self.args)
         return Basic._seq_subs(self, old, new)
 
-    def _eval_expand_basic(self, *args):
-        return None
-
     def _eval_evalf(self, prec):
         # Lookup mpmath function based on name
         fname = self.func.__name__
@@ -284,7 +276,7 @@ class Function(Basic):
 
     def count_ops(self, symbolic=True):
         #      f()             args
-        return 1   + Add(*[t.count_ops(symbolic) for t in self.args])
+        return 1 + Add(*[ t.count_ops(symbolic) for t in self.args ])
 
     def _eval_nseries(self, x, x0, n):
         assert len(self.args) == 1
@@ -294,9 +286,9 @@ class Function(Basic):
         if arg0 in [-oo, oo]:
             raise PoleError("Cannot expand around %s" % (arg))
         if arg0 is not S.Zero:
-            e = self.func(arg)
+            e = self
             e1 = e.expand()
-            if e==e1:
+            if e == e1:
                 #for example when e = sin(x+1) or e = sin(cos(x))
                 #let's try the general algorithm
                 term = e.subs(x, S.Zero)
@@ -310,7 +302,7 @@ class Function(Basic):
                     term = term.expand()
                     series += term
                 return series + C.Order(x**n, x)
-            return self.nseries(x, x0, n)
+            return e1.nseries(x, x0, n)
         l = []
         g = None
         for i in xrange(n+2):
@@ -325,15 +317,110 @@ class Function(Basic):
                 return False
         return True
 
-    def _eval_expand_complex(self, *args):
-        func = self.func(*[ a._eval_expand_complex(*args) for a in self.args ])
+    def _eval_expand_basic(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_basic'):
+                newterm = term._eval_expand_basic(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_power_exp(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_power_exp'):
+                newterm = term._eval_expand_power_exp(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_power_base(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_power_base'):
+                newterm = term._eval_expand_power_base(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_mul(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_mul'):
+                newterm = term._eval_expand_mul(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_multinomial(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_multinomail'):
+                newterm = term._eval_expand_multinomial(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_log(self, deep=True, **hints):
+        if not deep:
+            return self
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_log'):
+                newterm = term._eval_expand_log(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_complex(self, deep=True, **hints):
+        if deep:
+            func = self.func(*[ a.expand(deep, **hints) for a in self.args ])
+        else:
+            func = self.func(*self.args)
         return C.re(func) + S.ImaginaryUnit * C.im(func)
 
+    def _eval_expand_trig(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_trig'):
+                newterm = term._eval_expand_trig(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_func(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_func'):
+                newterm = term._eval_expand_func(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
     def _eval_rewrite(self, pattern, rule, **hints):
         if hints.get('deep', False):
             args = [ a._eval_rewrite(pattern, rule, **hints) for a in self ]
         else:
-            args = self.args[:]
+            args = self.args
 
         if pattern is None or isinstance(self.func, pattern):
             if hasattr(self, rule):
@@ -400,16 +487,17 @@ class WildFunction(Function, Atom):
         obj.name = name
         return obj
 
-    def matches(pattern, expr, repl_dict={}, evaluate=False):
-        for p,v in repl_dict.items():
-            if p==pattern:
-                if v==expr: return repl_dict
+    def matches(self, expr, repl_dict={}, evaluate=False):
+        if self in repl_dict:
+            if repl_dict[self] == expr:
+                return repl_dict
+            else:
                 return None
-        if pattern.nargs is not None:
-            if pattern.nargs != expr.nargs:
+        if self.nargs is not None:
+            if not hasattr(expr,'nargs') or self.nargs != expr.nargs:
                 return None
         repl_dict = repl_dict.copy()
-        repl_dict[pattern] = expr
+        repl_dict[self] = expr
         return repl_dict
 
     @classmethod
@@ -422,15 +510,16 @@ class WildFunction(Function, Atom):
 
 class Derivative(Basic):
     """
-    Carries out differentation of the given expression with respect to symbols.
+    Carries out differentiation of the given expression with respect to symbols.
 
     expr must define ._eval_derivative(symbol) method that returns
-    the differentation result or None.
+    the differentiation result or None.
 
     Examples:
 
     Derivative(Derivative(expr, x), y) -> Derivative(expr, x, y)
     Derivative(expr, x, 3)  -> Derivative(expr, x, x, x)
+
     """
 
     is_Derivative   = True
@@ -449,6 +538,7 @@ class Derivative(Basic):
         The second example shows why we don't return a list, but a generator,
         so that the code that calls _symbolgen can return earlier for special
         cases, like x.diff(x, 10**6).
+
         """
         last_s = sympify(symbols[len(symbols)-1])
         for i in xrange(len(symbols)):
@@ -476,12 +566,17 @@ class Derivative(Basic):
         symbols = Derivative._symbolgen(*symbols)
         if expr.is_commutative:
             assumptions["commutative"] = True
-        if assumptions.has_key("evaluate"):
+        if "evaluate" in assumptions:
             evaluate = assumptions["evaluate"]
             del assumptions["evaluate"]
         else:
             evaluate = False
         if not evaluate and not isinstance(expr, Derivative):
+            symbols = list(symbols)
+            if len(symbols) == 0:
+                # We make a special case for 0th derivative, because there
+                # is no good way to unambiguously print this.
+                return expr
             obj = Basic.__new__(cls, expr, *symbols, **assumptions)
             return obj
         unevaluated_symbols = []
@@ -511,12 +606,16 @@ class Derivative(Basic):
         if s not in self.symbols:
             obj = self.expr.diff(s)
             if isinstance(obj, Derivative):
-                return Derivative(obj.expr, *(obj.symbols+self.symbols))
+                return Derivative(obj.expr, *(self.symbols+obj.symbols))
             return Derivative(obj, *self.symbols)
         return Derivative(self.expr, *((s,)+self.symbols), **{'evaluate': False})
 
-    def doit(self):
-        return Derivative(self.expr, *self.symbols,**{'evaluate': True})
+    def doit(self, **hints):
+        expr = self.expr
+        if hints.get('deep', True):
+            expr = expr.doit(**hints)
+        hints['evaluate'] = True
+        return Derivative(expr, *self.symbols, **hints)
 
     @property
     def expr(self):
@@ -529,32 +628,29 @@ class Derivative(Basic):
     def _eval_subs(self, old, new):
         if self==old:
             return new
-        return Derivative(self.args[0]._eval_subs(old, new), *self.args[1:], **{'evaluate': True})
+        return Derivative(*map(lambda x: x._eval_subs(old, new), self.args), **{'evaluate': True})
 
-    def matches(pattern, expr, repl_dict={}, evaluate=False):
+    def matches(self, expr, repl_dict={}, evaluate=False):
         # this method needs a cleanup.
 
-        #print "?   :",pattern, expr, repl_dict, evaluate
-        #if repl_dict:
-        #    return repl_dict
-        for p,v in repl_dict.items():
-            if p==pattern:
-                if v==expr: return repl_dict
+        if self in repl_dict:
+            if repl_dict[self] == expr:
+                return repl_dict
+            else:
                 return None
-        assert isinstance(pattern, Derivative)
         if isinstance(expr, Derivative):
-            if len(expr.symbols) == len(pattern.symbols):
-                    #print "MAYBE:",pattern, expr, repl_dict, evaluate
-                    return Basic.matches(pattern, expr, repl_dict, evaluate)
-        #print "NONE:",pattern, expr, repl_dict, evaluate
+            if len(expr.symbols) == len(self.symbols):
+                    #print "MAYBE:",self, expr, repl_dict, evaluate
+                return Basic.matches(self, expr, repl_dict, evaluate)
+        #print "NONE:",self, expr, repl_dict, evaluate
         return None
-        #print pattern, expr, repl_dict, evaluate
+        #print self, expr, repl_dict, evaluate
         stop
-        if pattern.nargs is not None:
-            if pattern.nargs != expr.nargs:
+        if self.nargs is not None:
+            if self.nargs != expr.nargs:
                 return None
         repl_dict = repl_dict.copy()
-        repl_dict[pattern] = expr
+        repl_dict[self] = expr
         return repl_dict
 
     def _eval_lseries(self, x, x0):
@@ -581,23 +677,20 @@ class Lambda(Function):
     Lambda((x, y, ...), expr).
 
     A simple example:
-        >>> from sympy import Symbol
-        >>> x = Symbol('x')
+        >>> from sympy import Lambda
+        >>> from sympy.abc import x
         >>> f = Lambda(x, x**2)
         >>> f(4)
         16
 
     For multivariate functions, use:
-        >>> x = Symbol('x')
-        >>> y = Symbol('y')
-        >>> z = Symbol('z')
-        >>> t = Symbol('t')
-        >>> f2 = Lambda(x,y,z,t,x+y**z+t**z)
-        >>> f2(1,2,3,4)
+        >>> from sympy.abc import y, z, t
+        >>> f2 = Lambda(x, y, z, t, x + y**z + t**z)
+        >>> f2(1, 2, 3, 4)
         73
 
     Multivariate functions can be curries for partial applications:
-        >>> sum2numbers = Lambda(x,y,x+y)
+        >>> sum2numbers = Lambda(x, y, x+y)
         >>> sum2numbers(1,2)
         3
         >>> plus1 = sum2numbers(1)
@@ -605,9 +698,6 @@ class Lambda(Function):
         4
 
     A handy shortcut for lots of arguments:
-        >>> from sympy import *
-        >>> var('x y z')
-        (x, y, z)
         >>> p = x, y, z
         >>> f = Lambda(p, x + y*z)
         >>> f(*p)
@@ -625,11 +715,6 @@ class Lambda(Function):
         obj.nargs = len(args)-1
         return obj
 
-    @classmethod
-    @deprecated
-    def canonize(cls, *args):
-        return cls.eval(*args)
-
     @classmethod
     def eval(cls,*args):
         obj = Basic.__new__(cls, *args)
@@ -651,9 +736,8 @@ class Lambda(Function):
         This supports partial application.
 
         Example:
-            >>> from sympy import Symbol
-            >>> x = Symbol('x')
-            >>> y = Symbol('y')
+            >>> from sympy import Lambda
+            >>> from sympy.abc import x, y
             >>> f = Lambda(x, x**2)
             >>> f.apply(4)
             16
@@ -700,55 +784,293 @@ class Lambda(Function):
            #     return True
         return False
 
-@vectorize(0,1,2)
-def diff(f, x, times = 1, evaluate=True):
-    """Differentiate f with respect to x
+@vectorize(0)
+def diff(f, *symbols, **kwargs):
+    """
+    Differentiate f with respect to symbols.
 
-    It's just a wrapper to unify .diff() and the Derivative class,
-    it's interface is similar to that of integrate()
+    This is just a wrapper to unify .diff() and the Derivative class; its
+    interface is similar to that of integrate().  You can use the same
+    shortcuts for multiple variables as with Derivative.  For example,
+    diff(f(x), x, x, x) and diff(f(x), x, 3) both return the third derivative
+    of f(x).
+
+    You can pass evaluate=False to get an unevaluated Derivative class.  Note
+    that if there are 0 symbols (such as diff(f(x), x, 0), then the result will
+    be the function (the zeroth derivative), even if evaluate=False.
+
+    This function is vectorized, so you can pass a list for the arguments and
+    each argument will be mapped to each element of the list.  For a single
+    symbol, you can just pass the symbol normally.  For multiple symbols,
+    pass each group in a tuple.  For example, do diff(f(x, y), [x, y]) to get
+    the derivatives of f(x, y) with respect to x and with respect to y, and
+    diff(f(x, y), [(x, x), (y, y)]) to get the derivatives of f(x, y) with
+    respect to x twice and with respect to y twice.  You can also mix tuples
+    and single symbols.
+
+    Examples:
+    >>> from sympy import sin, cos, Function, diff
+    >>> from sympy.abc import x, y
+    >>> f = Function('f')
+
+    >>> diff(sin(x), x)
+    cos(x)
+    >>> diff(f(x), x, x, x)
+    D(f(x), x, x, x)
+    >>> diff(f(x), x, 3)
+    D(f(x), x, x, x)
+    >>> diff(sin(x)*cos(y), x, 2, y, 2)
+    cos(y)*sin(x)
+
+    >>> diff(f(x, y), [x, y])
+    [D(f(x, y), x), D(f(x, y), y)]
+    >>> diff(f(x, y), [(x, x), (y, y)])
+    [D(f(x, y), x, x), D(f(x, y), y, y)]
+    >>> diff(f(x, y), [(x, 2), y])
+    [D(f(x, y), x, x), D(f(x, y), y)]
+
+    >>> type(diff(sin(x), x))
+    cos
+    >>> type(diff(sin(x), x, evaluate=False))
+    <class 'sympy.core.function.Derivative'>
+    >>> type(diff(sin(x), x, 0))
+    sin
+    >>> type(diff(sin(x), x, 0, evaluate=False))
+    sin
+
+    See Also
+    http://documents.wolfram.com/v5/Built-inFunctions/AlgebraicComputation/Calculus/D.html
 
-    see http://documents.wolfram.com/v5/Built-inFunctions/AlgebraicComputation/Calculus/D.html
     """
 
-    return Derivative(f,x,times, **{'evaluate':evaluate})
+    # @vectorize(1) won't handle symbols in the way that we want, so we have to
+    # write the for loop manually.
+    if hasattr(symbols[0], '__iter__'):
+        retlist = []
+        for i in symbols[0]:
+            if hasattr(i, '__iter__'):
+                retlist.append(Derivative(f, *i, **kwargs))
+            else:
+                retlist.append(Derivative(f, i, **kwargs))
+        return retlist
+
+    kwargs.setdefault('evaluate', True)
+    return Derivative(f,*symbols, **kwargs)
 
 @vectorize(0)
-def expand(e, **hints):
+def expand(e, deep=True, power_base=True, power_exp=True, mul=True, \
+           log=True, multinomial=True, basic=True, **hints):
     """
-    Expand an expression using hints.
+    Expand an expression using methods given as hints.
+
+    Hints are applied with arbitrary order so your code shouldn't
+    depend on the way hints are passed to this method.
+
+    Hints evaluated unless explicitly set to False are:
+      basic, log, multinomial, mul, power_base, and power_exp
+    The following hints are supported but not applied unless set to True:
+      complex, func, and trig.
+
+    basic is a generic keyword for methods that want to be expanded
+    automatically.  For example, Integral uses expand_basic to expand the
+    integrand.  If you want your class expand methods to run automatically and
+    they don't fit one of the already automatic methods, wrap it around
+    _eval_expand_basic.
+
+    If deep is set to True, things like arguments of functions are
+    recursively expanded.  Use deep=False to only expand on the top
+    level.
+
+    Also see expand_log, expand_mul, expand_complex, expand_trig,
+    and expand_func, which are wrappers around those expansion methods.
+
+    >>> from sympy import cos, exp
+    >>> from sympy.abc import x, y, z
+
+    mul - Distributes multiplication over addition.
+    >>> (y*(x + z)).expand(mul=True)
+    x*y + y*z
+
+    complex - Split an expression into real and imaginary parts.
+    >>> (x+y).expand(complex=True)
+    I*im(x) + I*im(y) + re(x) + re(y)
+    >>> cos(x).expand(complex=True)
+    cos(re(x))*cosh(im(x)) - I*sin(re(x))*sinh(im(x))
+
+    power_exp - Expand addition in exponents into multiplied bases.
+    >>> exp(x+y).expand(power_exp=True)
+    exp(x)*exp(y)
+    >>> (2**(x+y)).expand(power_exp=True)
+    2**x*2**y
+
+    power_base - Split powers of multiplied bases.
+    >>> ((x*y)**z).expand(power_base=True)
+    x**z*y**z
+
+    log - Pull out power of an argument as a coefficient and split logs products
+    into sums of logs.  Note that these only work if the arguments of the log
+    function have the proper assumptions: the arguments must be positive and the
+    exponents must be real.
+    >>> from sympy import log, symbols
+    >>> log(x**2*y).expand(log=True)
+    log(y*x**2)
+    >>> x, y = symbols('xy', positive=True)
+    >>> log(x**2*y).expand(log=True)
+    2*log(x) + log(y)
+
+    trig - Do trigonometric expansions.
+    >>> cos(x+y).expand(trig=True)
+    cos(x)*cos(y) - sin(x)*sin(y)
+
+    func - Expand other functions.
+    >>> from sympy import gamma
+    >>> gamma(x+1).expand(func=True)
+    x*gamma(x)
+
+    multinomial - Expand (x + y + ...)**n where n is a positive integer.
+    >>> ((x+y+z)**2).expand(multinomial=True)
+    2*x*y + 2*x*z + 2*y*z + x**2 + y**2 + z**2
+
+    You can shut off methods that you don't want.
+    >>> (exp(x+y)*(x+y)).expand()
+    x*exp(x)*exp(y) + y*exp(x)*exp(y)
+    >>> (exp(x+y)*(x+y)).expand(power_exp=False)
+    x*exp(x + y) + y*exp(x + y)
+    >>> (exp(x+y)*(x+y)).expand(mul=False)
+    (x + y)*exp(x)*exp(y)
+
+    Use deep=False to only expand on the top level.
+    >>> exp(x+exp(x+y)).expand()
+    exp(x)*exp(exp(x)*exp(y))
+    >>> exp(x+exp(x+y)).expand(deep=False)
+    exp(x)*exp(exp(x + y))
+
+    Note: because hints are applied in arbitrary order, some hints may
+    prevent expansion by other hints if they are applied first.  In
+    particular, mul may distribute multiplications and prevent log and
+    power_base from expanding them.  Also, if mul is applied before multinomial,
+    the expression might not be fully distributed.  The solution is to expand
+    with mul=False first, then run expand_mul if you need further expansion.
+
+    Examples:
+    >>> from sympy import expand_log, expand, expand_mul
+    >>> x, y, z = symbols('xyz', positive=True)
+
+    >> expand(log(x*(y+z))) # could be either one below
+    log(x*y + x*z)
+    log(x) + log(y + z)
+
+    >>> expand_log(log(x*y+x*z))
+    log(x*y + x*z)
+
+    >> expand(log(x*(y+z)), mul=False)
+    log(x) + log(y + z)
+
+
+    >> expand((x*(y+z))**x) # could be either one below
+    (x*y + x*z)**x
+    x**x*(y + z)**x
+
+    >>> expand((x*(y+z))**x, mul=False)
+    x**x*(y + z)**x
+
+
+    >> expand(x*(y+z)**2) # could be either one below
+    2*x*y*z + x*y**2 + x*z**2
+    x*(y + z)**2
+
+    >>> expand(x*(y+z)**2, mul=False)
+    x*(2*y*z + y**2 + z**2)
+
+    >>> expand_mul(_)
+    2*x*y*z + x*y**2 + x*z**2
 
-    This is just a wrapper around Basic.expand(), see it's docstring of for a
-    thourough docstring for this function. In isympy you can just type
-    Basic.expand? and enter.
     """
-    return sympify(e).expand(**hints)
+    hints['power_base'] = power_base
+    hints['power_exp'] = power_exp
+    hints['mul'] = mul
+    hints['log'] = log
+    hints['multinomial'] = multinomial
+    hints['basic'] = basic
+    return sympify(e).expand(deep=deep, **hints)
+
+# These are simple wrappers around single hints.  Feel free to add ones for
+# power_exp, power_base, multinomial, or basic if you need them.
+def expand_mul(expr, deep=True):
+    """
+    Wrapper around expand that only uses the mul hint.  See the expand
+    docstring for more information.
+
+    Example:
+    >>> from sympy import symbols, expand_mul, exp, log
+    >>> x, y = symbols('xy', positive=True)
+    >>> expand_mul(exp(x+y)*(x+y)*log(x*y**2))
+    x*exp(x + y)*log(x*y**2) + y*exp(x + y)*log(x*y**2)
+
+    """
+    return sympify(expr).expand(deep=deep, mul=True, power_exp=False,\
+    power_base=False, basic=False, multinomial=False, log=False)
+
+def expand_log(expr, deep=True):
+    """
+    Wrapper around expand that only uses the log hint.  See the expand
+    docstring for more information.
+
+    Example:
+    >>> from sympy import symbols, expand_log, exp, log
+    >>> x, y = symbols('xy', positive=True)
+    >>> expand_log(exp(x+y)*(x+y)*log(x*y**2))
+    (x + y)*(2*log(y) + log(x))*exp(x + y)
+
+    """
+    return sympify(expr).expand(deep=deep, log=True, mul=False,\
+    power_exp=False, power_base=False, multinomial=False, basic=False)
+
+def expand_func(expr, deep=True):
+    """
+    Wrapper around expand that only uses the func hint.  See the expand
+    docstring for more information.
+
+    Example:
+    >>> from sympy import expand_func, gamma
+    >>> from sympy.abc import x
+    >>> expand_func(gamma(x + 2))
+    x*(1 + x)*gamma(x)
+
+    """
+    return sympify(expr).expand(deep=deep, func=True, basic=False,\
+    log=False, mul=False, power_exp=False, power_base=False, multinomial=False)
+
+def expand_trig(expr, deep=True):
+    """
+    Wrapper around expand that only uses the trig hint.  See the expand
+    docstring for more information.
+
+    Example:
+    >>> from sympy import expand_trig, sin, cos
+    >>> from sympy.abc import x, y
+    >>> expand_trig(sin(x+y)*(x+y))
+    (x + y)*(cos(x)*sin(y) + cos(y)*sin(x))
+
+    """
+    return sympify(expr).expand(deep=deep, trig=True, basic=False,\
+    log=False, mul=False, power_exp=False, power_base=False, multinomial=False)
+
+def expand_complex(expr, deep=True):
+    """
+    Wrapper around expand that only uses the complex hint.  See the expand
+    docstring for more information.
+
+    Example:
+    >>> from sympy import expand_complex, I, im, re
+    >>> from sympy.abc import z
+    >>> expand_complex(z**(2*I))
+    I*im(z**(2*I)) + re(z**(2*I))
+
+    """
+    return sympify(expr).expand(deep=deep, complex=True, basic=False,\
+    log=False, mul=False, power_exp=False, power_base=False, multinomial=False)
 
 
-# /cyclic/
-import basic as _
-_.Derivative    = Derivative
-_.FunctionClass = FunctionClass
-del _
-
-import add as _
-_.FunctionClass = FunctionClass
-del _
-
-import mul as _
-_.FunctionClass = FunctionClass
-_.WildFunction  = WildFunction
-del _
-
-import operations as _
-_.Lambda        = Lambda
-_.WildFunction  = WildFunction
-del _
-
-import symbol as _
-_.Function      = Function
-_.WildFunction  = WildFunction
-del _
-
-import numbers as _
-_.FunctionClass = FunctionClass
-del _
+from sympify import sympify
+from add    import Add
