@@ -1,21 +1,16 @@
-import math
+from basic import Atom, SingletonMeta, S, Basic
+from decorators import _sympifyit
+from cache import Memoizer, cacheit, clear_cache
 import sympy.mpmath as mpmath
 import sympy.mpmath.libmpf as mlib
-from sympy.mpmath.libmpf import mpf_abs
 import sympy.mpmath.libmpc as mlibc
-from sympy.mpmath.libelefun import mpf_pow, mpf_pi, phi_fixed
+from sympy.mpmath.libelefun import mpf_pow, mpf_pi, mpf_e, phi_fixed
 import decimal
 
 rnd = mlib.round_nearest
 
-from basic import Basic, Atom, S, C, SingletonMeta, Memoizer, MemoizerArg
-from sympify import _sympify, SympifyError, _sympifyit
-from power import integer_nthroot
-
-# from mul import Mul   /cyclic/
-# from power import Pow /cyclic/
-# from function import FunctionClass    /cyclic/
 
+# TODO: we should use the warnings module
 _errdict = {"divide": False}
 def seterr(divide=False):
     """
@@ -24,17 +19,19 @@ def seterr(divide=False):
     divide == True .... raise an exception
     divide == False ... return nan
     """
-    _errdict["divide"] = divide
+    if _errdict["divide"] != divide:
+        clear_cache()
+        _errdict["divide"] = divide
 
 # (a,b) -> gcd(a,b)
 _gcdcache = {}
 
 # TODO caching with decorator, but not to degrade performance
 def igcd(a, b):
-    """Computes integer greates common divisor of two numbers.
+    """Computes integer greatest common divisor of two numbers.
 
        The algorithm is based on the well known Euclid's algorithm. To
-       improve speed, igcd() has its own caching mechanizm implemented.
+       improve speed, igcd() has its own caching mechanism implemented.
     """
     try:
         return _gcdcache[(a,b)]
@@ -61,6 +58,7 @@ def ilcm(a, b):
 def igcdex(a, b):
     """Returns x, y, g such that g = x*a + y*b = gcd(a, b).
 
+       >>> from sympy.core.numbers import igcdex
        >>> igcdex(2, 3)
        (-1, 1, 1)
        >>> igcdex(10, 12)
@@ -164,7 +162,8 @@ class Number(Atom):
     is_Number = True
 
     def __new__(cls, *obj):
-        if len(obj)==1: obj=obj[0]
+        if len(obj)==1:
+            obj=obj[0]
         if isinstance(obj, (int, long)):
             return Integer(obj)
         if isinstance(obj,tuple) and len(obj)==2:
@@ -176,7 +175,7 @@ class Number(Atom):
         raise TypeError("expected str|int|long|float|Decimal|Number object but got %r" % (obj))
 
     def _as_mpf_val(self, prec):
-        """Evaluate to mpf tuple accurate to at least prec bits"""
+        """Evaluation of mpf tuple accurate to at least prec bits."""
         raise NotImplementedError('%s needs ._as_mpf_val() method' % \
             (self.__class__.__name__))
 
@@ -229,6 +228,9 @@ class Real(Number):
         Real(3.5)   .... 3.5 (the 3.5 was converted from a python float)
         Real("3.0000000000000005")
 
+        Real((1,3,0,2)) # mpmath tuple: (-1)**1 * 3 * 2**0; 3 has 2 bits
+        -3.00000000000000
+
     Notes:
     ======
         - Real(x) with x being a Python int/long will return Integer(x)
@@ -279,7 +281,15 @@ class Real(Number):
         if isinstance(num, (str, decimal.Decimal)):
             _mpf_ = mlib.from_str(str(num), prec, rnd)
         elif isinstance(num, tuple) and len(num) == 4:
-            _mpf_ = num
+            if type(num[1]) is str:
+                # it's a hexadecimal (coming from a pickled object)
+                # assume that it is in standard form
+                num = list(num)
+                num[1] = long(num[1], 16)
+                _mpf_ = tuple(num)
+            else:
+                _mpf_ = mpmath.mpf(
+                    S.NegativeOne ** num[0] * num[1] * 2 ** num[2])._mpf_
         else:
             _mpf_ = mpmath.mpf(num)._mpf_
         if not num:
@@ -310,21 +320,22 @@ class Real(Number):
     def __neg__(self):
         return Real._new(mlib.mpf_neg(self._mpf_), self._prec)
 
+    @_sympifyit('other', NotImplemented)
     def __mul__(self, other):
-        try:
-            other = _sympify(other)
-        except SympifyError:
-            return NotImplemented
         if isinstance(other, Number):
             rhs, prec = other._as_mpf_op(self._prec)
             return Real._new(mlib.mpf_mul(self._mpf_, rhs, prec, rnd), prec)
         return Number.__mul__(self, other)
 
+    @_sympifyit('other', NotImplemented)
+    def __mod__(self, other):
+        if isinstance(other, Number):
+            rhs, prec = other._as_mpf_op(self._prec)
+            return Real._new(mlib.mpf_mod(self._mpf_, rhs, prec, rnd), prec)
+        return Number.__mod__(self, other)
+
+    @_sympifyit('other', NotImplemented)
     def __add__(self, other):
-        try:
-            other = _sympify(other)
-        except SympifyError:
-            return NotImplemented
         if (other is S.NaN) or (self is NaN):
             return S.NaN
         if isinstance(other, Number):
@@ -367,6 +378,8 @@ class Real(Number):
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return False
             return other.__eq__(self)
+        if isinstance(other, FunctionClass): #cos as opposed to cos(x)
+            return False
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
             return bool(mlib.mpf_eq(self._mpf_, other._as_mpf_val(self._prec)))
@@ -380,6 +393,8 @@ class Real(Number):
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return True
             return other.__ne__(self)
+        if isinstance(other, FunctionClass): #cos as opposed to cos(x)
+            return True
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
             return bool(not mlib.mpf_eq(self._mpf_, other._as_mpf_val(self._prec)))
@@ -454,6 +469,7 @@ class Rational(Number):
 
     Examples
     ========
+        >>> from sympy import Rational
         >>> Rational(3)
         3
         >>> Rational(1,2)
@@ -485,6 +501,7 @@ class Rational(Number):
         3
         >>> r.q
         4
+
     """
     is_real = True
     is_integer = False
@@ -494,17 +511,21 @@ class Rational(Number):
 
     is_Rational = True
 
-    @Memoizer(type, (int, long, str, 'Integer'), MemoizerArg((int, long, 'Integer', type(None)), name="q"))
+    @cacheit
     def __new__(cls, p, q = None):
         if q is None:
-            if isinstance(p, str):
+            if isinstance(p, basestring):
                 p, q = _parse_rational(p)
+            elif isinstance(p, Rational):
+                return p
             else:
                 return Integer(p)
+        q = int(q)
+        p = int(p)
         if q==0:
             if p==0:
                 if _errdict["divide"]:
-                    raise Exception("Indeterminate 0/0")
+                    raise ValueError("Indeterminate 0/0")
                 else:
                     return S.NaN
             if p<0: return S.NegativeInfinity
@@ -519,8 +540,8 @@ class Rational(Number):
         if q==1: return Integer(p)
         if p==1 and q==2: return S.Half
         obj = Basic.__new__(cls)
-        obj.p = int(p)
-        obj.q = int(q)
+        obj.p = p
+        obj.q = q
         #obj._args = (p, q)
         return obj
 
@@ -548,6 +569,15 @@ class Rational(Number):
             return Rational(self.p * other.p, self.q * other.q)
         return Number.__mul__(self, other)
 
+    @_sympifyit('other', NotImplemented)
+    def __mod__(self, other):
+        if isinstance(other, Rational):
+            n = (self.p*other.q) // (other.p*self.q)
+            return Rational(self.p*other.q - n*other.p*self.q, self.q*other.q)
+        if isinstance(other, Real):
+            return self.evalf() % other
+        return Number.__mod__(self, other)
+
     # TODO reorder
     @_sympifyit('other', NotImplemented)
     def __add__(self, other):
@@ -568,8 +598,8 @@ class Rational(Number):
         return Number.__add__(self, other)
 
     def _eval_power(b, e):
+        if (e is S.NaN): return S.NaN
         if isinstance(e, Number):
-            if (e is S.NaN): return S.NaN
             if isinstance(e, Real):
                 return b._eval_evalf(e._prec) ** e
             if e.is_negative:
@@ -577,7 +607,13 @@ class Rational(Number):
                 ne = -e
                 if (ne is S.One):
                     return Rational(b.q, b.p)
-                return Rational(b.q, b.p) ** ne
+                if b < 0:
+                    if e.q != 1:
+                        return -(S.NegativeOne) ** ((e.p % e.q) / S(e.q)) * Rational(b.q, -b.p) ** ne
+                    else:
+                        return S.NegativeOne ** ne * Rational(b.q, -b.p) ** ne
+                else:
+                    return Rational(b.q, b.p) ** ne
             if (e is S.Infinity):
                 if b.p > b.q:
                     # (3/2)**oo -> oo
@@ -607,6 +643,9 @@ class Rational(Number):
     def _as_mpf_val(self, prec):
         return mlib.from_rational(self.p, self.q, prec, rnd)
 
+    def _mpmath_(self, prec, rnd):
+        return mpmath.make_mpf(mlib.from_rational(self.p, self.q, prec, rnd))
+
     def __abs__(self):
         return Rational(abs(self.p), self.q)
 
@@ -621,7 +660,7 @@ class Rational(Number):
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return False
             return other.__eq__(self)
-        if isinstance(self, Number) and isinstance(other, FunctionClass):
+        if isinstance(other, FunctionClass): #cos as opposed to cos(x)
             return False
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
@@ -639,10 +678,12 @@ class Rational(Number):
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return True
             return other.__ne__(self)
+        if isinstance(other, FunctionClass): #cos as opposed to cos(x)
+            return True
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(not mlib.feq(self._as_mpf_val(other._prec), other._mpf_))
+                return bool(not mlib.mpf_eq(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p!=other.p or self.q!=other.q)
 
         return True     # Rational != non-Number
@@ -681,7 +722,7 @@ class Rational(Number):
             try: f[p] += -e
             except KeyError: f[p] = -e
 
-        if len(f)>1 and f.has_key(1): del f[1]
+        if len(f)>1 and 1 in f: del f[1]
         return f
 
     def as_numer_denom(self):
@@ -764,27 +805,27 @@ class Integer(Rational):
     def _as_mpf_val(self, prec):
         return mlib.from_int(self.p)
 
+    def _mpmath_(self, prec, rnd):
+        return mpmath.make_mpf(self._as_mpf_val(prec))
+
     # TODO caching with decorator, but not to degrade performance
     @int_trace
     def __new__(cls, i):
         try:
             return _intcache[i]
         except KeyError:
-            # The most often situation is when Integers are created from Python
-            # int or long
-            if isinstance(i, (int, long)):
-                obj = Basic.__new__(cls)
-                obj.p = i
-                _intcache[i] = obj
-                return obj
-
-            # Also, we seldomly need the following to work:
-            # UC: Integer(Integer(4))   <-- sympify('4')
-            elif isinstance(i, Integer):
-                return i
-
+            # We only work with well-behaved integer types. This converts, for
+            # example, numpy.int32 instances.
+            ival = int(i)
+            if ival == 0: obj = S.Zero
+            elif ival == 1: obj = S.One
+            elif ival == -1: obj = S.NegativeOne
             else:
-                raise ValueError('invalid argument for Integer: %r' % (i,))
+                obj = Basic.__new__(cls)
+                obj.p = ival
+
+            _intcache[i] = obj
+            return obj
 
     def __getnewargs__(self):
         return (self.p,)
@@ -903,10 +944,10 @@ class Integer(Rational):
     def _eval_is_odd(self):
         return bool(self.p % 2)
 
-    def _eval_power(base, exp):
+    def _eval_power(b, e):
         """
-        Tries to do some simplifications on base ** exp, where base is
-        an instance of Rational
+        Tries to do some simplifications on b ** e, where b is
+        an instance of Integer
 
         Returns None if no further simplifications can be done
 
@@ -914,97 +955,73 @@ class Integer(Rational):
         we try to find the simplest possible representation, so that
           - 4**Rational(1,2) becomes 2
           - (-4)**Rational(1,2) becomes 2*I
+        We will
         """
-
-        MAX_INT_FACTOR = 4294967296 # Prevent from factorizing too big integers
-        if not isinstance(exp, Number):
-            c,t = base.as_coeff_terms()
-            if exp.is_even and isinstance(c, Number) and c < 0:
-                return (-c * Mul(*t)) ** exp
-        if exp is S.NaN: return S.NaN
-        if isinstance(exp, Real):
-            return base._eval_evalf(exp._prec) ** exp
-        if exp.is_negative:
-            # (3/4)**-2 -> (4/3)**2
-            ne = -exp
-            if ne is S.One:
-                return Rational(1, base.p)
-            return Rational(1, base.p) ** ne
-        if exp is S.Infinity:
-            if base.p > 1:
-                # (3)**oo -> oo
-                return S.Infinity
-            if base.p < -1:
-                # (-3)**oo -> oo + I*oo
-                return S.Infinity + S.Infinity * S.ImaginaryUnit
-            return S.Zero
-        if isinstance(exp, Integer):
-            # (4/3)**2 -> 4**2 / 3**2
-            return Integer(base.p ** exp.p)
-        if exp == S.Half and base.is_negative:
-            # sqrt(-2) -> I*sqrt(2)
-            return S.ImaginaryUnit * ((-base)**exp)
-        if isinstance(exp, Rational):
-            # 4**Rational(1,2) -> 2
+        if e is S.NaN: return S.NaN
+        if b is S.One: return S.One
+        if b is S.NegativeOne: return
+        if e is S.Infinity:
+            if b.p > S.One: return S.Infinity
+            if b.p == -1: return S.NaN
+            # cases 0, 1 are done in their respective classes
+            return S.Infinity + S.ImaginaryUnit * S.Infinity
+        if not isinstance(e, Number):
+            # simplify when exp is even
+            # (-2) ** k --> 2 ** k
+            c,t = b.as_coeff_terms()
+            if e.is_even and isinstance(c, Number) and c < 0:
+                return (-c * Mul(*t)) ** e
+        if not isinstance(e, Rational): return
+        if e is S.Half and b < 0:
+            # we extract I for this special case since everyone is doing so
+            return S.ImaginaryUnit * Pow(-b, e)
+        if e < 0:
+            # invert base and change sign on exponent
+            ne = -e
+            if b < 0:
+                if e.q != 1:
+                    return -(S.NegativeOne) ** ((e.p % e.q) / S(e.q)) * Rational(1, -b) ** ne
+                else:
+                    return (S.NegativeOne) ** ne * Rational(1, -b) ** ne
+            else:
+                return Rational(1, b.p) ** ne
+        # see if base is a perfect root, sqrt(4) --> 2
+        x, xexact = integer_nthroot(abs(b.p), e.q)
+        if xexact:
+            # if it's a perfect root we've finished
+            result = Integer(x ** abs(e.p))
+            if b < 0: result *= (-1)**e
+            return result
+        # The following is an algorithm where we collect perfect roots
+        # from the factors of base
+        if b > 4294967296:
+            # Prevent from factorizing too big integers
+            return None
+        dict = b.factors()
+        out_int = 1
+        sqr_int = 1
+        sqr_gcd = 0
+        sqr_dict = {}
+        for prime,exponent in dict.iteritems():
+            exponent *= e.p
+            div_e = exponent // e.q
+            div_m = exponent % e.q
+            if div_e > 0:
+                out_int *= prime**div_e
+            if div_m > 0:
+                sqr_dict[prime] = div_m
+        for p,ex in sqr_dict.iteritems():
+            if sqr_gcd == 0:
+                sqr_gcd = ex
+            else:
+                sqr_gcd = igcd(sqr_gcd, ex)
+        for k,v in sqr_dict.iteritems():
+            sqr_int *= k**(v // sqr_gcd)
+        if sqr_int == b.p and out_int == 1:
             result = None
-            x, xexact = integer_nthroot(abs(base.p), exp.q)
-            if xexact:
-                res = Integer(x ** abs(exp.p))
-                if exp >= 0:
-                    result = res
-                else:
-                    result = 1/res
-            else:
-                if base > MAX_INT_FACTOR:
-                    for i in xrange(2, exp.q//2 + 1): #OLD CODE
-                        if exp.q % i == 0:
-                            x, xexact = integer_nthroot(abs(base.p), i)
-                            if xexact:
-                                result = Integer(x)**(exp * i)
-                                break
-                    # Try to get some part of the base out, if exponent > 1
-                    if exp.p > exp.q:
-                        i = exp.p // exp.q
-                        r = exp.p % exp.q
-                        result = base**i * base**Rational(r, exp.q)
-
-                    return
-
-                dict = base.factors()
-                out_int = 1
-                sqr_int = 1
-                sqr_gcd = 0
-                sqr_dict = {}
-                for prime,exponent in dict.iteritems():
-                    exponent *= exp.p
-                    div_e = exponent // exp.q
-                    div_m = exponent % exp.q
-                    if div_e > 0:
-                        out_int *= prime**div_e
-                    if div_m > 0:
-                        sqr_dict[prime] = div_m
-                for p,ex in sqr_dict.iteritems():
-                    if sqr_gcd == 0:
-                        sqr_gcd = ex
-                    else:
-                        sqr_gcd = igcd(sqr_gcd, ex)
-                for k,v in sqr_dict.iteritems():
-                    sqr_int *= k**(v // sqr_gcd)
-                if sqr_int == base.p and out_int == 1:
-                    result = None
-                else:
-                    result = out_int * Pow(sqr_int , Rational(sqr_gcd, exp.q))
-
-            if result is not None:
-                if base.is_positive:
-                    return result
-                elif exp.q == 2:
-                    return result * S.ImaginaryUnit
-                else:
-                    return result * ((-1)**exp)
-            else:
-                if exp == S.Half and base.is_negative:
-                    return S.ImaginaryUnit * Pow(-base, exp)
+        else:
+            result = out_int * Pow(sqr_int , Rational(sqr_gcd, e.q))
+        return result
 
     def _eval_is_prime(self):
         if self.p < 0:
@@ -1082,9 +1099,6 @@ class One(Integer):
     def __neg__():
         return S.NegativeOne
 
-    def _eval_power(b, e):
-        return b
-
     def _eval_order(self, *symbols):
         return
 
@@ -1196,6 +1210,28 @@ class Infinity(Rational):
         import sage.all as sage
         return sage.oo
 
+    def __gt__(a, b):
+        if b is S.Infinity:
+            return False
+        return True
+
+    def __lt__(a, b):
+        return False
+
+    def __ge__(a, b):
+        return True
+
+    def __le__(a, b):
+        if b is S.Infinity:
+            return True
+        return False
+
+    def __mod__(self, other):
+        return S.NaN
+
+    __rmod__ = __mod__
+
+
 class NegativeInfinity(Rational):
     __metaclass__ = SingletonMeta
 
@@ -1246,6 +1282,23 @@ class NegativeInfinity(Rational):
     def _as_mpf_val(self, prec):
         return mlib.fninf
 
+    def __gt__(a, b):
+        return False
+
+    def __lt__(a, b):
+        if b is S.NegativeInfinity:
+            return False
+        return True
+
+    def __ge__(a, b):
+        if b is S.NegativeInfinity:
+            return True
+        return False
+
+    def __le__(a, b):
+        return True
+
+
 class NaN(Rational):
     __metaclass__ = SingletonMeta
 
@@ -1400,7 +1453,7 @@ class Exp1(NumberSymbol):
         return S.Exp1
 
     def _as_mpf_val(self, prec):
-        return mlib.fe(prec)
+        return mpf_e(prec)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls,Integer):
@@ -1453,7 +1506,7 @@ class GoldenRatio(NumberSymbol):
     def _as_mpf_val(self, prec):
         return mlib.from_man_exp(phi_fixed(prec+10), -prec-10)
 
-    def _eval_expand_func(self, *args):
+    def _eval_expand_func(self, deep=True, **hints):
         return S.Half + S.Half*S.Sqrt(5)
 
     def approximation_interval(self, number_cls):
@@ -1566,37 +1619,6 @@ class ImaginaryUnit(Atom):
         import sage.all as sage
         return sage.I
 
-# /cyclic/
-import basic as _
-_.Number    = Number
-_.Integer   = Integer
-_.Rational  = Rational
-_.Real      = Real
-del _
-
-import add as _
-_.Number    = Number
-del _
-
-import mul as _
-_.Number    = Number
-_.Integer   = Integer
-_.Real      = Real
-del _
-
-import power as _
-_.Number    = Number
-_.Rational  = Rational
-_.Integer   = Integer
-del _
-
-import sympify as _
-_.Integer   = Integer
-_.Real      = Real
-del _
-
-# ----
-
 _intcache[0] = S.Zero
 _intcache[1] = S.One
 _intcache[-1]= S.NegativeOne
@@ -1613,3 +1635,9 @@ Basic.singleton['GoldenRatio'] = GoldenRatio
 Basic.singleton['EulerGamma'] = EulerGamma
 Basic.singleton['Catalan'] = Catalan
 
+from basic import Basic, Atom, S, C, SingletonMeta
+from cache import Memoizer
+from sympify import _sympify, SympifyError
+from function import FunctionClass
+from power import Pow, integer_nthroot
+from mul import Mul
