@@ -1,16 +1,10 @@
-
-from basic import Basic, S, C
+from basic import Basic
 from sympify import _sympify
 from cache import cacheit
+from symbol import Symbol, Wild
 
 from sympy import mpmath
 
-from symbol import Symbol, Wild, Temporary
-# from numbers import Number, Rational, Integer     /cyclic/
-# from add import Add   /cyclic/
-# from mul import Mul   /cyclic/
-
-from math import exp as _exp
 from math import log as _log
 
 def integer_nthroot(y, n):
@@ -19,10 +13,12 @@ def integer_nthroot(y, n):
     and a boolean indicating whether the result is exact (that is,
     whether x**n == y).
 
+    >>> from sympy import integer_nthroot
     >>> integer_nthroot(16,2)
     (4, True)
     >>> integer_nthroot(26,2)
     (5, False)
+
     """
     if y < 0: raise ValueError("y must be nonnegative")
     if n < 1: raise ValueError("n must be positive")
@@ -73,19 +69,19 @@ class Pow(Basic):
     __slots__ = ['is_commutative']
 
     @cacheit
-    def __new__(cls, a, b, **assumptions):
-        a = _sympify(a)
+    def __new__(cls, b, e, **assumptions):
         b = _sympify(b)
+        e = _sympify(e)
         if assumptions.get('evaluate') is False:
-            return Basic.__new__(cls, a, b, **assumptions)
-        if b is S.Zero:
+            return Basic.__new__(cls, b, e, **assumptions)
+        if e is S.Zero:
             return S.One
-        if b is S.One:
-            return a
-        obj = a._eval_power(b)
+        if e is S.One:
+            return b
+        obj = b._eval_power(e)
         if obj is None:
-            obj = Basic.__new__(cls, a, b, **assumptions)
-            obj.is_commutative = (a.is_commutative and b.is_commutative)
+            obj = Basic.__new__(cls, b, e, **assumptions)
+            obj.is_commutative = (b.is_commutative and e.is_commutative)
         return obj
 
     @property
@@ -203,10 +199,11 @@ class Pow(Basic):
                 return True
 
     def _eval_subs(self, old, new):
-        if self==old: return new
-        if isinstance(old, self.__class__) and self.base==old.base:
-            coeff1,terms1 = self.exp.as_coeff_terms()
-            coeff2,terms2 = old.exp.as_coeff_terms()
+        if self == old:
+            return new
+        if old.func is self.func and self.base == old.base:
+            coeff1, terms1 = self.exp.as_coeff_terms()
+            coeff2, terms2 = old.exp.as_coeff_terms()
             if terms1==terms2: return new ** (coeff1/coeff2) # (x**(2*y)).subs(x**(3*y),z) -> z**(2/3*y)
         if old.func is C.exp:
             coeff1,terms1 = old.args[0].as_coeff_terms()
@@ -226,39 +223,67 @@ class Pow(Basic):
         from sympy.functions.elementary.complexes import conjugate as c
         return c(self.base)**self.exp
 
-    def _eval_expand_complex(self, *args):
-        if self.exp.is_Integer:
-            exp = self.exp
-            re, im = self.base.as_real_imag()
-            if exp >= 0:
-                base = re + S.ImaginaryUnit*im
+    def _eval_expand_basic(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_basic'):
+                newterm = term._eval_expand_basic(deep=deep, **hints)
             else:
-                mag = re**2 + im**2
-                base = re/mag - S.ImaginaryUnit*(im/mag)
-                exp = -exp
-            return (base**exp).expand()
-        elif self.exp.is_Rational:
-            # NOTE: This is not totally correct since for x**(p/q) with
-            #       x being imaginary there are actually q roots, but
-            #       only a single one is returned from here.
-            re, im = self.base.as_real_imag()
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
 
-            r = (re**2 + im**2)**S.Half
-            t = C.atan2(im, re)
-
-            rp, tp = r**self.exp, t*self.exp
-
-            return rp*C.cos(tp) + rp*C.sin(tp)*S.ImaginaryUnit
+    def _eval_expand_power_exp(self, deep=True, *args, **hints):
+        """a**(n+m) -> a**n*a**m"""
+        if deep:
+            b = self.base.expand(deep=deep, **hints)
+            e = self.exp.expand(deep=deep, **hints)
         else:
-            return C.re(self) + S.ImaginaryUnit*C.im(self)
+            b = self.base
+            e = self.exp
+        if e.is_Add:
+            expr = 1
+            for x in e.args:
+                if deep:
+                    x = x.expand(deep=deep, **hints)
+                expr *= (self.base**x)
+            return expr
+        return b**e
 
-    def _eval_expand_basic(self):
-        """
-        (a*b)**n -> a**n * b**n
-        (a+b+..) ** n -> a**n + n*a**(n-1)*b + .., n is positive integer
-        """
-        b = self.base._eval_expand_basic()
-        e = self.exp._eval_expand_basic()
+    def _eval_expand_power_base(self, deep=True, **hints):
+        """(a*b)**n -> a**n * b**n"""
+        b = self.base
+        if deep:
+            e = self.exp.expand(deep=deep, **hints)
+        else:
+            e = self.exp
+        if b.is_Mul:
+            if deep:
+                return Mul(*(Pow(t.expand(deep=deep, **hints), e)\
+                for t in b.args))
+            else:
+                return Mul(*[Pow(t, e) for t in b.args])
+        else:
+            return b**e
+
+    def _eval_expand_mul(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_mul'):
+                newterm = term._eval_expand_mul(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_multinomial(self, deep=True, **hints):
+        """(a+b+..) ** n -> a**n + n*a**(n-1)*b + .., n is nonzero integer"""
+        if deep:
+            b = self.base.expand(deep=deep, **hints)
+            e = self.exp.expand(deep=deep, **hints)
+        else:
+            b = self.base
+            e = self.exp
 
         if b is None:
             base = self.base
@@ -359,7 +384,13 @@ class Pow(Basic):
                 if n == 2:
                     return Add(*[f*g for f in base.args for g in base.args])
                 else:
-                    return Mul(base, Pow(base, n-1).expand()).expand()
+                    multi = (base**(n-1))._eval_expand_multinomial(deep=False)
+                    if multi.is_Add:
+                        return Add(*[f*g for f in base.args for g in base.args])
+                    else:
+                        return Add(*[f*multi for f in base.args])
+        elif exp.is_Integer and exp.p < 0 and base.is_Add:
+            return 1 / Pow(base, -exp.p)._eval_expand_multinomial(deep=False)
         elif exp.is_Add and base.is_Number:
             #  a + b      a  b
             # n      --> n  n  , where n, a, b are Numbers
@@ -376,6 +407,68 @@ class Pow(Basic):
         else:
             return result
 
+    def _eval_expand_log(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_log'):
+                newterm = term._eval_expand_log(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_complex(self, deep=True, **hints):
+        if self.exp.is_Integer:
+            exp = self.exp
+            re, im = self.base.as_real_imag()
+            if exp >= 0:
+                base = re + S.ImaginaryUnit*im
+            else:
+                mag = re**2 + im**2
+                base = re/mag - S.ImaginaryUnit*(im/mag)
+                exp = -exp
+            return (base**exp).expand()
+        elif self.exp.is_Rational:
+            # NOTE: This is not totally correct since for x**(p/q) with
+            #       x being imaginary there are actually q roots, but
+            #       only a single one is returned from here.
+            re, im = self.base.as_real_imag()
+
+            r = (re**2 + im**2)**S.Half
+            t = C.atan2(im, re)
+
+            rp, tp = r**self.exp, t*self.exp
+
+            return rp*C.cos(tp) + rp*C.sin(tp)*S.ImaginaryUnit
+        else:
+            if deep:
+                hints['complex'] = False
+                return C.re(self.expand(deep, **hints)) + \
+                S.ImaginaryUnit*C.im(self. expand(deep, **hints))
+            else:
+                return C.re(self) + S.ImaginaryUnit*C.im(self)
+            return C.re(self) + S.ImaginaryUnit*C.im(self)
+
+    def _eval_expand_trig(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_trig'):
+                newterm = term._eval_expand_trig(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
+    def _eval_expand_func(self, deep=True, **hints):
+        sargs, terms = self.args, []
+        for term in sargs:
+            if hasattr(term, '_eval_expand_func'):
+                newterm = term._eval_expand_func(deep=deep, **hints)
+            else:
+                newterm = term
+            terms.append(newterm)
+        return self.new(*terms)
+
     def _eval_derivative(self, s):
         dbase = self.base.diff(s)
         dexp = self.exp.diff(s)
@@ -411,33 +504,33 @@ class Pow(Basic):
 
     def as_numer_denom(self):
         base, exp = self.as_base_exp()
-        c,t = exp.as_coeff_terms()
-        n,d = base.as_numer_denom()
-        negate = False
-        if exp.is_integer != True:
-            if d.is_negative == True:
-                # Roots need to take care that negative denominators behave
-                # differently than the rest of the complex plane.
-                negate = True
-            elif d.is_negative is None:
-                # Can make no conclusions.
-                return self, S(1)
-        if c.is_negative == True:
-            exp = -exp
-            n,d = d,n
-        num = n ** exp
-        den = d ** exp
-        if negate:
-            num = -num
-        return num, den
+        n, d = base.as_numer_denom()
+        if exp.is_Integer:
+            if exp.is_negative:
+                n, d = d, n
+                exp = -exp
+            return n ** exp, d ** exp
+        elif exp.is_Rational:
+            if d.is_negative is None:
+                # we won't split up the base
+                if exp.is_negative:
+                    #return d * base ** -exp, n
+                    return S.One, base ** -exp
+                else:
+                    return self, S.One
+            if d.is_negative:
+                n = -n
+                d = -d
+            if exp.is_negative:
+                n, d = d, n
+                exp = -exp
+            return n ** exp, d ** exp
+        # unprocessed Real and NumberSymbol
+        return self, S.One
 
-    def matches(pattern, expr, repl_dict={}, evaluate=False):
+    def matches(self, expr, repl_dict={}, evaluate=False):
         if evaluate:
-            pat = pattern
-            for old,new in repl_dict.items():
-                pat = pat.subs(old, new)
-            if pat!=pattern:
-                return pat.matches(expr, repl_dict)
+            return self.subs(repl_dict).matches(expr, repl_dict)
 
         expr = _sympify(expr)
         b, e = expr.as_base_exp()
@@ -445,21 +538,23 @@ class Pow(Basic):
         # special case, pattern = 1 and expr.exp can match to 0
         if expr is S.One:
             d = repl_dict.copy()
-            d = pattern.exp.matches(S.Zero, d, evaluate=False)
+            d = self.exp.matches(S.Zero, d)
             if d is not None:
                 return d
 
         d = repl_dict.copy()
-        d = pattern.base.matches(b, d, evaluate=False)
+        d = self.base.matches(b, d)
         if d is None:
             return None
 
-        d = pattern.exp.matches(e, d, evaluate=True)
+        d = self.exp.subs(d).matches(e, d)
         if d is None:
-            return Basic.matches(pattern, expr, repl_dict, evaluate)
+            return Basic.matches(self, expr, repl_dict, evaluate)
         return d
 
     def _eval_nseries(self, x, x0, n):
+        from sympy import powsimp, collect
+
         def geto(e):
             "Returns the O(..) symbol, or None if there is none."
             if e.is_Order:
@@ -481,6 +576,7 @@ class Pow(Basic):
             2
             >>> getn(1+x)
             >>>
+
             """
             o = geto(e)
             if o is None:
@@ -492,21 +588,21 @@ class Pow(Basic):
                 if o.is_Pow:
                     return o.args[1]
                 n, d = o.as_numer_denom()
-                if isinstance(d, log):
+                if d.func is log:
                     # i.e. o = x**2/log(x)
                     if n.is_Symbol:
                         return Integer(1)
                     if n.is_Pow:
                         return n.args[1]
 
-            raise Exception("Unimplemented")
+            raise NotImplementedError()
 
         base, exp = self.args
         if exp.is_Integer:
             if exp > 0:
                 # positive integer powers are easy to expand, e.g.:
                 # sin(x)**4 = (x-x**3/3+...)**4 = ...
-                return (base.nseries(x, x0, n) ** exp).expand()
+                return (base.nseries(x, x0, n) ** exp)._eval_expand_multinomial(deep = False)
             elif exp == -1:
                 # this is also easy to expand using the formula:
                 # 1/(1 + x) = 1 + x + x**2 + x**3 ...
@@ -521,6 +617,7 @@ class Pow(Basic):
                         p = p.nseries(y, x0, n)
                         p = p.subs(y, -1/log(x))
                         return p
+
                 base = base.nseries(x, x0, n)
                 if base.has(log(x)):
                     # we need to handle the log(x) singularity:
@@ -534,43 +631,43 @@ class Pow(Basic):
                         return p
                 prefactor = base.as_leading_term(x)
                 # express "rest" as: rest = 1 + k*x**l + ... + O(x**n)
-                rest = ((base-prefactor)/prefactor).expand()
+                rest = ((base-prefactor)/prefactor)._eval_expand_mul()
                 if rest == 0:
                     # if prefactor == w**4 + x**2*w**4 + 2*x*w**4, we need to
                     # factor the w**4 out using collect:
-                    from sympy import collect
                     return 1/collect(prefactor, x)
                 if rest.is_Order:
-                    return ((1+rest)/prefactor).expand()
-                if not rest.has(x):
-                    return 1/(prefactor*(rest+1))
+                    return (1+rest)/prefactor
                 n2 = getn(rest)
                 if n2 is not None:
                     n = n2
 
-                from sympy import collect
                 term2 = collect(rest.as_leading_term(x), x)
                 k, l = Wild("k"), Wild("l")
                 r = term2.match(k*x**l)
                 k, l = r[k], r[l]
-                if l.is_Integer and l>0:
-                    l = int(l)
+                if l.is_Rational and l>0:
+                    pass
                 elif l.is_number and l>0:
-                    l = float(l)
+                    l = l.evalf()
                 else:
-                    raise Exception("Not implemented")
+                    raise NotImplementedError()
 
-                s = 1
-                m = 1
-                while l * m < n:
-                    s += ((-rest)**m).expand()
-                    m += 1
-                r = (s/prefactor).expand()
+                from sympy.functions import ceiling
+                terms = [1/prefactor]
+                for m in xrange(1,ceiling(n/l)):
+                    new_term = terms[-1]*(-rest)
+                    if new_term.is_Pow:
+                        new_term = new_term._eval_expand_multinomial(deep = False)
+                    else:
+                        new_term = new_term._eval_expand_mul(deep = False)
+                    terms.append(new_term)
+                r = Add(*terms)
                 if n2 is None:
                     # Append O(...) because it is not included in "r"
                     from sympy import O
                     r += O(x**n)
-                return r
+                return powsimp(r, deep=True, combine='exp')
             else:
                 # negative powers are rewritten to the cases above, for example:
                 # sin(x)**(-4) = 1/( sin(x)**4) = ...
@@ -586,7 +683,7 @@ class Pow(Basic):
             return sympy.exp(exp*sympy.log(base)).nseries(x, x0, n)
 
         if base == x:
-            return self
+            return powsimp(self, deep=True, combine='exp')
 
         order = C.Order(x**n, x)
         x = order.symbols[0]
@@ -596,7 +693,8 @@ class Pow(Basic):
         exp = C.exp
         if e.has(x):
             return exp(e * ln(b)).nseries(x, x0, n)
-        if b==x: return self
+        if b==x:
+            return self
         b0 = b.limit(x,0)
         if b0 is S.Zero or b0.is_unbounded:
             lt = b.as_leading_term(x)
@@ -608,6 +706,7 @@ class Pow(Basic):
                 # bs -> lt + rest -> lt * (1 + (bs/lt - 1))
                 return (lt**e * ((bs/lt).expand()**e).nseries(x,
                         x0, n-e)).expand() + order
+
             return bs**e+order
         o2 = order * (b0**-e)
         # b -> b0 + (b-b0) -> b0 * (1 + (b/b0-1))
@@ -659,16 +758,7 @@ class Pow(Basic):
         return self.args[0]._sage_() ** self.args[1]._sage_()
 
 
-# /cyclic/
-import basic as _
-_.Pow =     Pow
-del _
-
-import mul as _
-_.Pow =     Pow
-del _
-
-import numbers as _
-_.Pow =     Pow
-del _
-
+from basic import Basic, S, C
+from add import Add
+from numbers import Integer
+from mul import Mul
