@@ -1,5 +1,5 @@
 from sympy import abc, Function, Symbol, Wild, Derivative, sin, cos, Real, \
-        Rational, exp, I, Integer, diff, Mul, var, oo
+        Rational, exp, I, Integer, diff, Mul, var, oo, S, Add
 from sympy.utilities.pytest import XFAIL
 
 
@@ -130,8 +130,10 @@ def test_functions():
     q = Wild('q')
 
     f = cos(5*x)
+    notf = x
     assert f.match(p*cos(q*x)) == {p: 1, q: 5}
     assert f.match(p*g) == {p: 1, g: cos(5*x)}
+    assert notf.match(g) == None
 
 @XFAIL
 def test_functions_X1():
@@ -215,9 +217,9 @@ def test_match_bug2():
     assert (p+q+r).subs(res) == x+y
 
 def test_match_bug3():
-     x,a,b = map(Symbol, 'xab')
-     p = Wild('p')
-     assert (b*x*exp(a*x)).match(x*exp(p*x)) == None
+    x,a,b = map(Symbol, 'xab')
+    p = Wild('p')
+    assert (b*x*exp(a*x)).match(x*exp(p*x)) == None
 
 def test_match_bug4():
     x = Symbol('x')
@@ -275,6 +277,9 @@ def test_exclude():
     q = Wild('q', exclude=[x])
     r = Wild('r', exclude=[sin,y])
 
+    assert sin(x).match(r) == None
+    assert cos(y).match(r) == None
+
     e = 3*x**2 + y*x + a
     assert e.match(p*x**2 + q*x + r) == {p: 3, q: y, r: a}
 
@@ -333,3 +338,5 @@ def test_combine_inverse():
     assert Mul._combine_inverse(x*I*y, y*I) == x
     assert Mul._combine_inverse(oo*I*y, y*I) == oo
     assert Mul._combine_inverse(oo*I*y, oo*I) == y
+    assert Add._combine_inverse(oo, oo) == S(0)
+    assert Add._combine_inverse(oo*I, oo*I) == S(0)
