@@ -1,5 +1,5 @@
-
 from sympy.core import Function, S, C, Basic, sympify, pi
+from sympy.utilities.decorator import deprecated
 
 ###############################################################################
 ###################### HURWITZ GENERALIZED ZETA FUNCTION ######################
@@ -10,7 +10,7 @@ class zeta(Function):
     nargs = (1, 2)
 
     @classmethod
-    def canonize(cls, z, a=S.One):
+    def eval(cls, z, a=S.One):
         z, a = map(sympify, (z, a))
 
         if a.is_Number:
@@ -38,6 +38,8 @@ class zeta(Function):
                     elif z.is_even:
                         B, F = C.bernoulli(z), C.Factorial(z)
                         zeta = 2**(z-1) * abs(B) * pi**z / F
+                    else:
+                        return
 
                     if a.is_negative:
                         return zeta + C.harmonic(abs(a), z)
@@ -52,7 +54,7 @@ class dirichlet_eta(Function):
     nargs = 1
 
     @classmethod
-    def canonize(cls, s):
+    def eval(cls, s):
         if s == 1:
             return C.log(2)
         else:
