@@ -1,23 +1,27 @@
-
-from sympy.core import Basic, S, C, Symbol, Wild, Pow, sympify, diff
+from sympy.core import Basic, S, C, Symbol, Wild, Pow, Add, sympify, diff, oo
 
 from sympy.integrals.trigonometry import trigintegrate
 from sympy.integrals.deltafunctions import deltaintegrate
 from sympy.integrals.rationaltools import ratint
 from sympy.integrals.risch import heurisch
-from sympy.utilities import threaded
+from sympy.utilities import threaded, flatten
+from sympy.utilities.iterables import make_list
 from sympy.simplify import apart
 from sympy.series import limit
 from sympy.polys import Poly
 from sympy.solvers import solve
 from sympy.functions import DiracDelta, Heaviside, Piecewise
 from sympy.geometry import Curve
+from sympy.functions.elementary.piecewise import piecewise_fold
 
 class Integral(Basic):
     """Represents unevaluated integral."""
 
     def __new__(cls, function, *symbols, **assumptions):
-        function = sympify(function)
+        # Any embedded piecewise functions need to be brought out to the
+        # top level so that integration can go into piecewise mode at the
+        # earliest possible moment.
+        function = piecewise_fold(sympify(function))
 
         if function.is_Number:
             if function is S.NaN:
@@ -35,19 +39,25 @@ class Integral(Basic):
                     limits.append((V,None))
                     continue
                 elif isinstance(V, (tuple, list)):
+                    V = flatten(V)
+                    newsymbol = sympify(V[0])
                     if len(V) == 3:
-                        if isinstance(V[0], Symbol):
+                        if isinstance(newsymbol, Symbol):
                             nlim = map(sympify, V[1:])
-                            limits.append( (V[0], tuple(nlim) ))
+                            if V[1] is None:
+                                nlim[0] = None
+                            if V[2] is None:
+                                nlim[1] = None
+                            limits.append( (newsymbol, tuple(nlim) ))
                             continue
                     elif len(V) == 1 or (len(V) == 2 and V[1] is None):
-                        if isinstance(V[0], Symbol):
-                            limits.append((V[0],None))
+                        if isinstance(newsymbol, Symbol):
+                            limits.append((newsymbol,None))
                             continue
 
                 raise ValueError("Invalid integration variable or limits: %s" % str(symbols))
         else:
-            # no symbols provided -- let's compute full antiderivative
+            # no symbols provided -- let's compute full anti-derivative
             limits = [(symb,None) for symb in function.atoms(Symbol)]
 
             if not limits:
@@ -60,7 +70,13 @@ class Integral(Basic):
 
     def __getnewargs__(self):
         function, limits = self.args
-        return (function,) + limits
+        newlimits = []
+        for i in limits:
+            if i[1] == None:
+                newlimits.append((i[0]))
+            else:
+                newlimits.append((i[0], i[1][0], i[1][1]))
+        return (function,) + tuple(newlimits)
 
     @property
     def function(self):
@@ -128,22 +144,40 @@ class Integral(Basic):
         if not hints.get('integrals', True):
             return self
 
+        deep = hints.get('deep', True)
+
         function = self.function
+        if deep:
+            function = function.doit(**hints)
 
         for x,ab in self.limits:
             antideriv = self._eval_integral(function, x)
 
             if antideriv is None:
-                return self
+                newargs = (function, self.__getnewargs__()[1])
+                return self.new(*newargs)
             else:
                 if ab is None:
                     function = antideriv
                 else:
-                    a,b = ab
+                    a, b = ab
+                    if deep:
+                        if isinstance(a, Basic):
+                            a = a.doit(**hints)
+                        if isinstance(b, Basic):
+                            b = b.doit(**hints)
                     function = antideriv._eval_interval(x, a, b)
 
         return function
 
+    def _eval_expand_basic(self, deep=True, **hints):
+        from sympy import flatten
+        if not deep:
+            return self
+        else:
+            return Integral(self.function.expand(deep=deep, **hints),\
+            flatten(*self.limits))
+
     def _eval_derivative(self, sym):
         """Evaluate the derivative of the current Integral object.
         We follow these steps:
@@ -187,7 +221,7 @@ class Integral(Basic):
                integrate(diff(self.function, sym), (int_var, lower_limit, upper_limit))
 
     def _eval_integral(self, f, x):
-        """Calculate the antiderivative to the function f(x).
+        """Calculate the anti-derivative to the function f(x).
 
         This is a powerful function that should in theory be able to integrate
         everything that can be integrated. If you find something, that it
@@ -195,8 +229,8 @@ class Integral(Basic):
 
         (1) Simple heuristics (based on pattern matching and integral table):
 
-         - most frequently used functions (eg. polynomials)
-         - functions non-integrable by any of the following algorithms (eg.
+         - most frequently used functions (e.g. polynomials)
+         - functions non-integrable by any of the following algorithms (e.g.
            exp(-x**2))
 
         (2) Integration of rational functions:
@@ -236,7 +270,7 @@ class Integral(Basic):
             return f.integrate(x)
 
         # Piecewise antiderivatives need to call special integrate.
-        if isinstance(f,Piecewise):
+        if f.func is Piecewise:
             return f._eval_integral(x)
 
         # let's cut it short if `f` does not depend on `x`
@@ -252,14 +286,8 @@ class Integral(Basic):
         # since Integral(f=g1+g2+...) == Integral(g1) + Integral(g2) + ...
         # we are going to handle Add terms separately,
         # if `f` is not Add -- we only have one term
-        if not f.is_Add:
-            f = [f]
-
         parts = []
-
-        if isinstance(f, Basic):
-            f = f.args
-        for g in f:
+        for g in make_list(f, Add):
             coeff, g = g.as_independent(x)
 
             # g(x) = const
@@ -287,7 +315,7 @@ class Integral(Basic):
             #        poly(x)
             # g(x) = -------
             #        poly(x)
-            if g.is_fraction(x):
+            if g.is_rational_function(x):
                 parts.append(coeff * ratint(g, x))
                 continue
 
@@ -340,12 +368,99 @@ class Integral(Basic):
             if sym == old:
                 return self
             if limits is not None:
-                a, b, = limits
-                arg1.append((sym, a.subs(old, new), b.subs(old, new)))
+                if limits[0] is None:
+                    b = limits[1]
+                    arg1.append((sym, None, b.subs(old, new)))
+                elif limits[1] is None:
+                    a = limits[0]
+                    arg1.append((sym, a.subs(old, new), None))
+                else:
+                    a, b, = limits
+                    arg1.append((sym, a.subs(old, new), b.subs(old, new)))
             else:
                 arg1.append((sym, limits))
         return Integral(arg0, *arg1)
 
+    def as_sum(self, n, method="midpoint"):
+        """
+        Approximates the integral by a sum.
+
+        method ... one of: left, right, midpoint
+
+        This is basically just the rectangle method [1], the only difference is
+        where the function value is taken in each interval.
+
+        [1] http://en.wikipedia.org/wiki/Rectangle_method
+
+        **method = midpoint**:
+
+        Uses the n-order midpoint rule to evaluate the integral.
+
+        Midpoint rule uses rectangles approximation for the given area (e.g.
+        definite integral) of the function with heights equal to the point on
+        the curve exactly in the middle of each interval (thus midpoint
+        method). See [1] for more information.
+
+        Examples:
+
+            >>> from sympy import sqrt
+            >>> from sympy.abc import x
+            >>> from sympy.integrals import Integral
+            >>> e = Integral(sqrt(x**3+1), (x, 2, 10))
+            >>> e
+            Integral((1 + x**3)**(1/2), (x, 2, 10))
+            >>> e.as_sum(4, method="midpoint")
+            2*730**(1/2) + 4*7**(1/2) + 4*86**(1/2) + 6*14**(1/2)
+            >>> e.as_sum(4, method="midpoint").n()
+            124.164447891310
+            >>> e.n()
+            124.616199194723
+
+        **method=left**:
+
+        Uses the n-order rectangle rule to evaluate the integral, at each
+        interval the function value is taken at the left hand side of the
+        interval.
+
+        Examples:
+
+            >>> from sympy import sqrt
+            >>> from sympy.abc import x
+            >>> e = Integral(sqrt(x**3+1), (x, 2, 10))
+            >>> e
+            Integral((1 + x**3)**(1/2), (x, 2, 10))
+            >>> e.as_sum(4, method="left")
+            6 + 2*65**(1/2) + 2*217**(1/2) + 6*57**(1/2)
+            >>> e.as_sum(4, method="left").n()
+            96.8853618335341
+            >>> e.n()
+            124.616199194723
+
+        """
+
+        if len(self.args[1]) > 1:
+            raise NotImplementedError("Multidimensional midpoint rule not implemented yet")
+        if n <= 0:
+            raise ValueError("n must be > 0")
+        if n == oo:
+            raise NotImplementedError("Infinite summation not yet implemented")
+        sym, limits = self.args[1][0]
+        dx = (limits[1]-limits[0])/n
+        result = 0.
+        for i in range(n):
+            if method == "midpoint":
+                xi = limits[0] + i*dx + dx/2
+            elif method == "left":
+                xi = limits[0] + i*dx
+            elif method == "right":
+                xi = limits[0] + i*dx + dx
+            else:
+                raise NotImplementedError("Unknown method %s" % method)
+            result += self.args[0].subs(sym, xi)
+        return result*dx
+
+
+
 @threaded(use_add=False)
 def integrate(*args, **kwargs):
     """integrate(f, var, ...)
@@ -364,35 +479,37 @@ def integrate(*args, **kwargs):
        Several variables can be specified, in which case the result is multiple
        integration.
 
-       Also, if no var is specified at all, then full-antiderivative of f is
-       returned. This is equivalent of integrating f over all it's variables.
+       Also, if no var is specified at all, then the full anti-derivative of f is
+       returned. This is equivalent to integrating f over all its variables.
 
        Examples
 
-       >>> from sympy import *
-       >>> x, y = symbols('xy')
+       >>> from sympy import integrate, log
+       >>> from sympy.abc import a, x, y
 
        >>> integrate(x*y, x)
-       (1/2)*y*x**2
+       y*x**2/2
 
        >>> integrate(log(x), x)
        -x + x*log(x)
 
+       >>> integrate(log(x), (x, 1, a))
+       1 - a + a*log(a)
+
        >>> integrate(x)
-       (1/2)*x**2
+       x**2/2
 
        >>> integrate(x*y)
-       (1/4)*x**2*y**2
+       x**2*y**2/4
 
        See also the doctest of Integral._eval_integral(), which explains
        thoroughly the strategy that SymPy uses for integration.
 
     """
-    new_args = [sympify(arg) for arg in args]
-    integral = Integral(*new_args, **kwargs)
+    integral = Integral(*args, **kwargs)
 
     if isinstance(integral, Integral):
-        return integral.doit()
+        return integral.doit(deep = False)
     else:
         return integral
 
@@ -405,11 +522,11 @@ def line_integrate(field, curve, vars):
 
        Examples
        --------
-       >>> from sympy import *
-       >>> x, y, t = symbols('xyt')
+       >>> from sympy import Curve, line_integrate, E, ln
+       >>> from sympy.abc import x, y, t
        >>> C = Curve([E**t + 1, E**t - 1], (t, 0, ln(2)))
        >>> line_integrate(x + y, C, [x, y])
-       3*sqrt(2)
+       3*2**(1/2)
 
     """
     F = sympify(field)
@@ -437,6 +554,5 @@ def line_integrate(field, curve, vars):
         Ft = Ft.subs(var, _f)
     Ft = Ft * dldt**(S(1)/2)
 
-    integral = Integral(Ft, curve.limits).doit()
+    integral = Integral(Ft, curve.limits).doit(deep = False)
     return integral
-
