@@ -146,16 +146,6 @@ def test_issue565():
     assert integrate(-Rational(1)/2 * x * sin(n * pi * x/2), [x, -2, 0]) \
                                                                  == 2*cos(pi*n)/(pi*n)
 
-
-def test_rational_functions():
-    half = Rational(1,2)
-    integrate(1/(x**2+x+1), x) == -I*3**half*log(half + x - half*I*3**half)/3 +\
-                                   I*3**half*log(half + x + half*I*3**half)/3
-
-    integrate(1/(x**3+1), x) == log(1 + x)/3 - \
-        (Rational(1,6) - I*3**half/6)*log(half - x - I*3**half/2) - \
-        (Rational(1,6) + I*3**half/6)*log(half - x + I*3**half/2)
-
 def test_issue580():
     # definite integration of rational functions gives wrong answers
     assert NS(Integral(1/(x**2-8*x+17), (x, 2, 4))) == '1.10714871779409'
@@ -182,6 +172,9 @@ def test_issue853():
     assert integrate(f, x) == -cos(x)
     raises(ValueError, "integrate(f, 2*x)")
 
+def test_issue1417():
+    assert integrate(2**x - 2*x, x) == 2**x/log(2) - x**2
+
 def test_matrices():
     M = Matrix(2, 2, lambda i, j: (i+j+1)*sin((i+j+1)*x))
 
@@ -339,3 +332,80 @@ def test_subs5():
 def test_integration_variable():
     raises(ValueError, "Integral(exp(-x**2), 3)")
     raises(ValueError, "Integral(exp(-x**2), (3, -oo, oo))")
+
+def test_expand_integral():
+    assert Integral(cos(x**2)*(sin(x**2)+1),(x, 0, 1)).expand() == Integral(cos(x**2)*sin(x**2) + cos(x**2), (x, 0, 1))
+    assert Integral(cos(x**2)*(sin(x**2)+1),x).expand() == Integral(cos(x**2)*sin(x**2) + cos(x**2), x)
+
+def test_as_sum_midpoint1():
+    e = Integral(sqrt(x**3+1), (x, 2, 10))
+    assert e.as_sum(1, method="midpoint") == 8*217**(S(1)/2)
+    assert e.as_sum(2, method="midpoint") == 4*65**(S(1)/2) + 12*57**(S(1)/2)
+    assert e.as_sum(3, method="midpoint") == 8*217**(S(1)/2)/3 + \
+            8*3081**(S(1)/2)/27 + 8*52809**(S(1)/2)/27
+    assert e.as_sum(4, method="midpoint") == 2*730**(S(1)/2) + \
+            4*7**(S(1)/2) + 4*86**(S(1)/2) + 6*14**(S(1)/2)
+    assert abs(e.as_sum(4, method="midpoint").n() - e.n()) < 0.5
+
+    e = Integral(sqrt(x**3+y**3), (x, 2, 10), (y, 0, 10))
+    raises(NotImplementedError, "e.as_sum(4)")
+
+def test_as_sum_midpoint2():
+    e = Integral((x+y)**2, (x, 0, 1))
+    assert e.as_sum(1, method="midpoint").expand() == S(1)/4 + y + y**2
+    assert e.as_sum(2, method="midpoint").expand() == S(5)/16 + y + y**2
+    assert e.as_sum(3, method="midpoint").expand() == S(35)/108 + y + y**2
+    assert e.as_sum(4, method="midpoint").expand() == S(21)/64 + y + y**2
+
+def test_as_sum_left():
+    e = Integral((x+y)**2, (x, 0, 1))
+    assert e.as_sum(1, method="left").expand() == y**2
+    assert e.as_sum(2, method="left").expand() == S(1)/8 + y/2 + y**2
+    assert e.as_sum(3, method="left").expand() == S(5)/27 + 2*y/3 + y**2
+    assert e.as_sum(4, method="left").expand() == S(7)/32 + 3*y/4 + y**2
+
+def test_as_sum_right():
+    e = Integral((x+y)**2, (x, 0, 1))
+    assert e.as_sum(1, method="right").expand() == 1 + 2*y + y**2
+    assert e.as_sum(2, method="right").expand() == S(5)/8 + 3*y/2 + y**2
+    assert e.as_sum(3, method="right").expand() == S(14)/27 + 4*y/3 + y**2
+    assert e.as_sum(4, method="right").expand() == S(15)/32 + 5*y/4 + y**2
+
+def test_as_sum_raises():
+    e = Integral((x+y)**2, (x, 0, 1))
+    raises(ValueError, "e.as_sum(-1)")
+    raises(ValueError, "e.as_sum(0)")
+    raises(NotImplementedError, "e.as_sum(oo)")
+    raises(NotImplementedError, "e.as_sum(3, method='xxxx2')")
+
+def test_nested_doit():
+    e = Integral(Integral(x, x), x)
+    f = Integral(x, x, x)
+    assert e.doit() == f.doit()
+
+def test_issue1566():
+    # Allow only upper or lower limit evaluation
+    e = Integral(x**2, (x, None, 1))
+    f = Integral(x**2, (x, 1, None))
+    assert e.doit() == Rational(1, 3)
+    assert f.doit() == Rational(-1, 3)
+    assert Integral(x*y, (x, None, y)).subs(y, t) == Integral(x*t, (x, None, t))
+    assert Integral(x*y, (x, y, None)).subs(y, t) == Integral(x*t, (x, t, None))
+    assert integrate(x**2, (x, None, 1)) == Rational(1, 3)
+    assert integrate(x**2, (x, 1, None)) == Rational(-1, 3)
+    assert integrate("x**2", ("x", "1", None)) == Rational(-1, 3)
+
+def test_integral_reconstruct():
+    e = Integral(x**2, (x, -1, 1))
+    assert e == Integral(*e.args)
+
+
+def test_doit():
+    e = Integral(Integral(2*x), (x, 0, 1))
+    assert e.doit() == Rational(1, 3)
+
+@XFAIL
+def test_doit2():
+    e = Integral(Integral(2*x), (x, 0, 1))
+    # risch currently chokes on the contained integral.
+    assert e.doit(deep = False) == e
