@@ -306,7 +306,7 @@ def zzX_from_dict(f, l):
         if len(tail) == 1:
             tail = tail[0]
 
-        if coeffs.has_key(head):
+        if head in coeffs:
             coeffs[head][tail] = INT_TYPE(int(coeff))
         else:
             coeffs[head] = { tail : INT_TYPE(int(coeff)) }
@@ -1535,7 +1535,7 @@ def zzX_heu_gcd(f, g, **flags):
        The algorithm computes the polynomial GCD by evaluating polynomials
        f and g at certain points and computing (fast) integer GCD of those
        evaluations. The polynomial GCD is recovered from the integer image
-       by interpolation. The evaluation proces reduces f and g variable by
+       by interpolation. The evaluation process reduces f and g variable by
        variable into a large integer.  The final step  is to verify if the
        interpolated polynomial is the correct GCD. This gives cofactors of
        the input polynomials as a side effect (see zzX_cofactors).
@@ -2011,6 +2011,7 @@ def zzx_factor(f, **flags):
 
        Consider polynomial f = 2*x**4 - 2:
 
+       >>> from sympy.polys.integerpolys import zzx_factor
        >>> zzx_factor([2, 0, 0, 0, -2])
        (2, [([1, -1], 1), ([1, 1], 1), ([1, 0, 1], 1)])
 
@@ -2023,7 +2024,7 @@ def zzx_factor(f, **flags):
 
        By default, polynomials x**n - 1 and x**n + 1 are factored
        using cyclotomic decomposition to speedup computations. To
-       disable this behaviour set cyclotomic=False.
+       disable this behavior set cyclotomic=False.
 
        For more details on the implemented algorithm refer to:
 
@@ -2081,7 +2082,7 @@ def zzx_factor(f, **flags):
     return cont, sorted(factors, compare)
 
 def zzx_factor_sqf(f, **flags):
-    """Factor square-free (non-primitive) polyomials in Z[x].  """
+    """Factor square-free (non-primitive) polynomials in Z[x].  """
     cont, g = zzx_primitive(f)
 
     n = zzx_degree(g)
@@ -2572,6 +2573,7 @@ def zzX_factor(f):
 
        Consider polynomial f = 2*(x**2 - y**2):
 
+       >>> from sympy.polys.integerpolys import zzX_factor
        >>> f = [[2], [], [-2, 0, 0]]
 
        >>> zzX_factor(f)
@@ -2637,4 +2639,3 @@ def zzX_factor(f):
             return i
 
     return cont, sorted(factors, compare)
-
