@@ -1,4 +1,4 @@
-
+from sympy.core.symbol import Symbol
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core.basic import Basic, S
@@ -11,7 +11,7 @@ from sympy.polys.algorithms import poly_decompose, poly_sqf, poly_div
 from sympy.polys.factortools import poly_factors
 
 from sympy.ntheory import divisors
-from sympy.functions import exp
+from sympy.functions import exp, sqrt
 
 def roots_linear(f):
     """Returns a list of roots of a linear polynomial."""
@@ -31,84 +31,170 @@ def roots_quadratic(f):
         (-b - d) / (2*a),
     ]
 
-    from sympy.simplify import simplify
-
-    return [ simplify(r) for r in roots ]
+    return roots
 
 def roots_cubic(f):
-    """Returns a list of  roots of a cubic polynomial."""
-    if isinstance(f, (tuple, list)):
-        a, b, c, d = f
-    else:
-        a, b, c, d = f.iter_all_coeffs()
+    """Returns a list of roots of a cubic polynomial."""
+    one, a, b, c = f.as_monic().iter_all_coeffs()
 
-    b, c, d = b/a, c/a, d/a
+    if c is S.Zero:
+        x1, x2 = roots([1,a,b], multiple = True)
+        return [x1, S.Zero, x2]
 
-    p = c - b**2 / 3
-    q = d + (2*b**3 - 9*b*c) / 27
+    p = b - a**2/3
+    q = c - a*b/3 + 2*a**3/27
+
+    pon3 = p/3
+    aon3 = a/3
 
     if p is S.Zero:
         if q is S.Zero:
-            return [-b/3] * 3
+            return [-aon3] * 3
         else:
             u1 = q**Rational(1, 3)
+    elif q is S.Zero:
+        y1, y2 = roots([1, 0, p], multiple=True)
+        return [tmp - aon3 for tmp in [y1, S.Zero, y2]]
     else:
-        u1 = (q/2 + (q**2/4 + p**3/27)**S.Half)**Rational(1, 3)
+        u1 = (q/2 + (q**2/4 + pon3**3)**S.Half)**Rational(1, 3)
 
     coeff = S.ImaginaryUnit*3**S.Half / 2
 
     u2 = u1*(-S.Half + coeff)
     u3 = u1*(-S.Half - coeff)
 
-    roots = [
-        (p/(3*u1) - u1 - b/3),
-        (p/(3*u2) - u2 - b/3),
-        (p/(3*u3) - u3 - b/3),
+    soln = [
+        -u1 + pon3/u1 - aon3,
+        -u2 + pon3/u2 - aon3,
+        -u3 + pon3/u3 - aon3
     ]
 
-    return [ r.expand() for r in roots ]
+    return soln
 
 def roots_quartic(f):
     """Returns a list of roots of a quartic polynomial.
 
+    There are many references for solving quartic expressions available [1-5].
+    This reviewer has found that many of them require one to select from among
+    2 or more possible sets of solutions and that some solutions work when one
+    is searching for real roots but don't work when searching for complex roots
+    (though this is not always stated clearly). The following routine has been
+    tested and found to be correct for 0, 2 or 4 complex roots.
+
+    The quasisymmetric case solution[6] looks for quartics that have the form
+    x**4 + A*x**3 + B*x**2 + C*x + D = 0 where (C/A)**2 = D.
+
+    NOTE: There is not a single symbolic solution that is valid for all
+    possible values of A, B, C and D. There are 4 sets of solutions possible
+    based on the code below. These solutions (determined by the values of the
+    reduced quartic coefficients, a = B/A, b = C/A, c = D/A, and d = E/A, are:
+      1) f = c + a * (a**2 / 8 - b / 2) == 0
+      2) g = d - a * (a * (3 * a**2 / 256 - b / 16) + c / 4) = 0
+      3) if f != 0 and g !=0 but
+        a) p = -d + a*c/4 - b**2/12 = 0 and
+           q = b*d/3 + a*b*c/24 - c**2/8 - d*a**2/8 - b**3/108 >= 0
+           then u (see code) will be zero, otherwise
+        b) u != 0
+
+            Schematically, it looks like this:
+
+                f = 0    g = 0           f != 0 and g != 0
+                  |        |                   /   \
+                  |        |       p=0 and q>=0     p!=0 or q<0
+                  |        |       (i.e. u = 0     (i.e. u != 0)
+                  |        |            |                |
+                4 solns  4 solns      4 solns          4 solns
+                                             (default symbolic solution)
+
+    These branches do not count the special cases that have a particularly
+    simple form of the roots. Those special cases can often be solved by the
+    general procedure. In the test suite, there are tests for those cases,
+    and each of them marked with "general soln ok, too" in the code below
+    still gave the same for the tests.
+
+    Example::
+    >>> from sympy import var, Poly
+    >>> from sympy.polys.rootfinding import roots_quartic
+    >>> x = var('x')
+    >>> r = roots_quartic(Poly(x**4 -6*x**3 +17*x**2 -26*x +20, x))
+    >>> # 4 complex roots: 1+-I*sqrt(3), 2+-I
+    >>> sorted(str(tmp.evalf(n=2)) for tmp in r)
+    ['1.0 + 1.7*I', '1.0 - 1.7*I', '2.0 + I', '2.0 - 1.0*I']
+
     References:
-      http://planetmath.org/encyclopedia/GaloisTheoreticDerivationOfTheQuarticFormula.html
+      [1] http://mathforum.org/dr.math/faq/faq.cubic.equations.html
+      [2] http://en.wikipedia.org/wiki/Quartic_function#Summary_of_Ferrari.27s_method
+      [3] http://planetmath.org/encyclopedia/GaloisTheoreticDerivationOfTheQuarticFormula.html
+      [4] http://staff.bath.ac.uk/masjhd/JHD-CA.pdf
+      [5] http://www.albmath.org/files/Math_5713.pdf
+      [6] http://www.statemaster.com/encyclopedia/Quartic-equation
+          #Other_particular_case:_Quasi-symmetric_equations
+
     """
-    a, b, c, d, e = f.iter_all_coeffs()
 
-    # normalize
-    t = a
-    a = b/t
-    b = c/t
-    c = d/t
-    d = e/t
+    # normalized coefficients
+    one, a, b, c, d = f.as_monic().iter_all_coeffs()
 
-    p = -2*b
-    q = b**2 + a*c - 4*d
-    r = c**2 + a**2*d - a*b*c
+    if d is S.Zero: # general soln ok, too
+        return [S.Zero] + roots([1, a, b, c], multiple = True)
+    elif (c/a)**2 == d: # general solution ok, too
+        m = sqrt(d)
+        z = Symbol('z', dummy=True)
+        x = f.symbols[0]
+        z1, z2 = roots_quadratic(Poly(z**2+a*z+b-2*m, z))
+        f = Poly(x**2 - z*x + m, x)
+        return roots_quadratic(f.subs(z, z1)) + \
+               roots_quadratic(f.subs(z, z2))
+    else:
+        a2 = a ** 2
+        e = b - 3 * a2 / 8
+        f = c + a * (a2 / 8 - b / 2)
+        g = d - a * (a * (3 * a2 / 256 - b / 16) + c / 4)
+        aon4 = a / 4
+        ans = []
 
-    r = roots_cubic((1, p, q, r))
-
-    u = r[1] + r[2] - r[0]
-    v = (u**2 - 16*d)**S.Half
-    w = (a**2 - 4*r[0])**S.Half
-
-    A = (u + v) / 4
-    B = (u - v) / 4
-    C = (-a + w) / 2
-    D = (-a - w) / 2
-
-    E = (C**2 - 4*A)**S.Half
-    F = (D**2 - 4*B)**S.Half
-
-    roots = [
-        (C + E) / 2,
-        (C - E) / 2,
-        (D + F) / 2,
-        (D - F) / 2,
-    ]
-
-    return [ r.expand() for r in roots ]
+        if f.is_zero: # general solution not valid if f = 0.
+            y1, y2 = [tmp ** S.Half for tmp in
+                      roots([1, e, g], multiple = True)]
+            return [tmp - aon4 for tmp in [-y1, -y2, y1, y2]]
+        if g.is_zero: # general solution not valid if g = 0
+            y = [S.Zero] + roots([1, 0, e, f], multiple = True)
+            return [tmp - aon4 for tmp in y]
+        else:
+            TH = Rational(1, 3)
+            p = -e**2/12 - g
+            q = -e**3/108 + e*g/3 - f**2/8
+            root = sqrt(q**2/4 + p**3/27)
+            rr = [-q/2 + s*root for s in [1]] # in [1,-1], either will do, so pick 1
+            for r in rr:
+                uu = [r**TH] # solve(x**3-r,x), any one will do, so take primary
+                for u in uu:
+                    if u.is_zero:
+                        y = -5*e/6 + u - q**TH
+                    else:
+                        y = -5*e/6 + u - p/u/3
+                    w = sqrt(e + 2*y)
+                    # try to return the real solutions first if the
+                    # sign can be determined. The term tested is the
+                    # argument to `root` below.
+                    arg1 = 3*e + 2*y
+                    arg2 = 2*f/w
+                    if -(arg1 + arg2) > S.Zero:
+                        ss = [-1, 1]
+                    else:
+                        ss= [1, -1]
+                    for s in ss:
+                        root = sqrt(-(arg1 + s*arg2))
+                        # return the more negative root first;
+                        # note that t and s have opposite signs
+                        # in the formula: s=+/-1 while t=-/+1.
+                        if aon4 > S.Zero:
+                            tt = [-1, 1]
+                        else:
+                            tt = [1, -1]
+                        for t in tt:
+                            ans.append((s*w - t*root)/2 - aon4)
+    return ans
 
 def roots_binomial(f):
     """Returns a list of roots of a binomial polynomial."""
@@ -120,11 +206,12 @@ def roots_binomial(f):
     if alpha.is_number:
         alpha = alpha.expand(complex=True)
 
+
     roots, I = [], S.ImaginaryUnit
 
     for k in xrange(n):
         zeta = exp(2*k*S.Pi*I/n).expand(complex=True)
-        roots.append((alpha*zeta).expand())
+        roots.append((alpha*zeta).expand(power_base=False))
 
     return roots
 
@@ -157,10 +244,11 @@ def roots_rational(f):
     return zeros
 
 def roots(f, *symbols, **flags):
-    """Computes symbolic roots of an univariate polynomial.
+    """Computes symbolic roots of a univariate polynomial.
 
-       Given an univariate  polynomial f with symbolic coefficients,
-       returns a dictionary with its roots and their multiplicities.
+       Given a univariate polynomial f with symbolic coefficients (or
+       a list of the polynomial's coefficients), returns a dictionary
+       with its roots and their multiplicities.
 
        Only roots expressible via radicals will be returned.  To get
        a complete set of roots use RootOf class or numerical methods
@@ -176,26 +264,51 @@ def roots(f, *symbols, **flags):
        case of multiple roots.  However to get a tuple containing all
        those roots set the 'multiple' flag to True.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy import Poly, roots
+       >>> from sympy.abc import x, y
 
        >>> roots(x**2 - 1, x)
        {1: 1, -1: 1}
 
+       >>> p = Poly(x**2-1, x)
+       >>> roots(p)
+       {1: 1, -1: 1}
+
+       >>> m = Poly(x**2-y, x, y) # multivariate
+       >>> roots(Poly(m, x)) # same expr, but now univariate
+       {y**(1/2): 1, -y**(1/2): 1}
+
+       >>> roots([1, 0, -1]) # LIST of coefficients
+       {1: 1, -1: 1}
+
        >>> roots(x**2 - y, x)
        {y**(1/2): 1, -y**(1/2): 1}
 
     """
+
+    if isinstance(f, list):
+        if symbols:
+            raise SymbolsError("Redundant symbol(s) given.")
+        f = Poly(f, Symbol('x', dummy=True))
+
     if not isinstance(f, Poly):
-        f = Poly(f, *symbols)
-    elif symbols:
-        raise SymbolsError("Redundant symbols were given")
+        if len(symbols) > 1:
+            raise SymbolsError("Too many symbols given.")
+        f = Poly(f, *(symbols or S(f).atoms(Symbol)))
+
+    if len(symbols) > 1:
+        raise SymbolsError("Too many symbols given.")
 
     if f.is_multivariate:
-        raise MultivariatePolyError(f)
+        if len(symbols) == 1:
+            f = Poly(f, *symbols)
+        else:
+            raise MultivariatePolyError(f)
+
+    assert f.is_univariate
 
     def _update_dict(result, root, k):
-        if result.has_key(root):
+        if root in result:
             result[root] += k
         else:
             result[root] = k
@@ -251,7 +364,7 @@ def roots(f, *symbols, **flags):
             result += roots_quadratic(f)
         elif n == 3 and flags.get('cubics', True):
             result += roots_cubic(f)
-        elif n == 4 and flags.get('quartics', False):
+        elif n == 4 and flags.get('quartics', True):
             result += roots_quartic(f)
 
         return result
@@ -337,15 +450,16 @@ def roots(f, *symbols, **flags):
         return zeros
 
 def poly_root_factors(f, *symbols, **flags):
-    """Returns all factors of an univariate polynomial.
+    """Returns all factors of a univariate polynomial.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.rootfinding import poly_root_factors
+       >>> from sympy.abc import x, y
 
        >>> factors = poly_root_factors(x**2-y, x)
 
        >>> set(f.as_basic() for f in factors)
        set([x + y**(1/2), x - y**(1/2)])
+
     """
     if not isinstance(f, Poly):
         f = Poly(f, *symbols)
@@ -357,7 +471,7 @@ def poly_root_factors(f, *symbols, **flags):
     else:
         x = f.symbols[0]
 
-    if flags.has_key('multiple'):
+    if 'multiple' in flags:
         del flags['multiple']
 
     zeros = roots(f, **flags)
@@ -380,7 +494,7 @@ def poly_root_factors(f, *symbols, **flags):
 def poly_sturm(f, *symbols):
     """Computes the Sturm sequence of a given polynomial.
 
-       Given an univariate, square-free polynomial f(x) returns an
+       Given a univariate, square-free polynomial f(x) returns an
        associated Sturm sequence f_0(x), ..., f_n(x) defined by:
 
            f_0(x), f_1(x) = f(x), f'(x)
@@ -413,8 +527,9 @@ def poly_sturm(f, *symbols):
 def number_of_real_roots(f, *symbols, **flags):
     """Returns the number of distinct real roots of f in (inf, sup].
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.rootfinding import number_of_real_roots
+       >>> from sympy.abc import x, y
+       >>> from sympy import Poly, sqrt
 
        >>> f = Poly(x**2 - 1, x)
 
@@ -498,7 +613,7 @@ def number_of_real_roots(f, *symbols, **flags):
 _exact_roots_cache = {}
 
 def _exact_roots(f):
-    if _exact_roots_cache.has_key(f):
+    if f in _exact_roots_cache:
         zeros = _exact_roots_cache[f]
     else:
         exact, zeros = roots(f), []
@@ -511,7 +626,7 @@ def _exact_roots(f):
     return zeros
 
 class RootOf(Basic):
-    """Represents n-th root of an univariate polynomial. """
+    """Represents n-th root of a univariate polynomial. """
 
     def __new__(cls, f, index):
         if isinstance(f, RootsOf):
@@ -544,10 +659,10 @@ class RootOf(Basic):
         return self.poly.atoms(*args, **kwargs)
 
 class RootsOf(Basic):
-    """Represents all roots of an univariate polynomial.
+    """Represents all roots of a univariate polynomial.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy import roots, RootsOf
+       >>> from sympy.abc import x, y
 
        >>> roots = RootsOf(x**2 + 2, x)
 
@@ -607,7 +722,7 @@ class RootsOf(Basic):
 
 
 class RootSum(Basic):
-    """Represents a sum of all roots of an univariate polynomial. """
+    """Represents a sum of all roots of a univariate polynomial. """
 
     def __new__(cls, f, *args, **flags):
         if not hasattr(f, '__call__'):
@@ -649,4 +764,3 @@ class RootSum(Basic):
                 result += self.function(root)
 
             return result
-
