@@ -1,12 +1,13 @@
 from sympy import symbols, expand, sin, sqrt, re, im, I, Rational, Lambda, \
-        powsimp, raises, Integer, Symbol, Poly, RootsOf, RootSum, RootOf, S
+        separate, raises, Integer, Symbol, Poly, RootsOf, RootSum, RootOf, S
 
 from sympy.polys.monomial import monomial_lex_cmp, monomial_grlex_cmp, \
         monomial_grevlex_cmp, monomial_1_el_cmp
 
 from sympy.polys.algorithms import poly_groebner, poly_subresultants,    \
         poly_resultant, poly_half_gcdex, poly_gcdex, poly_gcd, poly_lcm, \
-        poly_div, poly_pdiv, poly_decompose, poly_sqf, poly_reduce
+        poly_div, poly_pdiv, poly_decompose, poly_sqf, poly_reduce, \
+        poly_discriminant
 
 from sympy.polys.rootfinding import poly_root_factors, roots_linear,  \
         roots_quadratic, roots_cubic, roots_quartic, roots_binomial, \
@@ -15,7 +16,10 @@ from sympy.polys.rootfinding import poly_root_factors, roots_linear,  \
 from sympy.polys.factortools import poly_factors, factors, factor
 
 from sympy.polys.polynomial import Poly, PolynomialError, \
-        CoefficientError, SymbolsError
+        CoefficientError, SymbolsError, MultivariatePolyError
+
+from sympy.utilities.pytest import skip
+from sympy.utilities import all
 
 a,b,c,d,x,y,z,u,v,t = symbols('abcdxyzuvt')
 
@@ -25,9 +29,26 @@ def test_monomial_cmp():
     assert monomial_grevlex_cmp((1,3,1), (1,2,2)) == 1
     assert monomial_1_el_cmp((2,0,1), (1,2,0)) == 1
 
+def test_poly_has():
+    f = x*y**2*z + I*x*y + x + 1
+    assert Poly(f, x).has(x) == True
+    assert Poly(f, x, y).has(x) == True
+    assert Poly(f, y).has(x) == True
+    assert Poly(f, x, y).has(z) == True
+    assert Poly(f, x, y).has(t) == False
+    assert Poly(f, x, y).has(I) == True
+    assert Poly(f, y).has(I) == True
+    assert Poly(f, x).has(I) == True
+
 def test_poly_basics():
     f = x*y**2*z + x*y + x + 1
 
+    raises(SymbolsError, "Poly(f, x, x)")
+    raises(PolynomialError, "Poly(((1, 2), (3, 4), (5, 6)), x)")
+    raises(CoefficientError, "Poly([(x, 3)], x)")
+    raises(PolynomialError, "Poly([(1, (3,2))], x)")
+    assert Poly([(1, (2,)), (2, (2,))], x) == Poly(3*x**2, x)
+    assert Poly([(1, (2,)), (-1, (2,))], x) == Poly(0, x)
     assert Poly(f, x).as_basic().expand() == f
     assert Poly(f, x, y).as_basic().expand() == f
     assert Poly(f, x, y, z).as_basic().expand() == f
@@ -46,6 +67,8 @@ def test_poly_basics():
 
     p = Poly(x**2 + x + 1, x)
 
+    assert Poly([1, 1, 1], x) == p
+
     assert Poly(p, y) == \
         Poly(((1 + x + x**2,), ((0,),)), y, order='grlex')
 
@@ -615,14 +638,24 @@ def test_poly_subresultants():
     g = 3*x**6+5*x**4-4*x**2-9*x+21
 
     assert poly_subresultants(f, g, x) == \
-        [Poly(f, x), Poly(g, x),
-         Poly(15*x**4 - 3*x**2 + 9,  x),
-         Poly(65*x**2 + 125*x - 245, x),
-         Poly(9326*x - 12300, x),
-         Poly(260708, x)]
+        (Poly(260708, x), [Poly(f, x), Poly(g, x),
+                           Poly(15*x**4 - 3*x**2 + 9,  x),
+                           Poly(65*x**2 + 125*x - 245, x),
+                           Poly(9326*x - 12300, x),
+                           Poly(260708, x)])
 
     assert poly_subresultants((x-1)**2, x**2-1, x) == \
-        [Poly((x-1)**2, x), Poly(x**2-1, x), Poly(2*x - 2, x)]
+        (Poly(0, x), [Poly((x-1)**2, x),
+                      Poly(x**2-1, x),
+                      Poly(2*x - 2, x)])
+
+    f = Poly(-x**3 + 5, x)
+    g = Poly((1 + 3*t)*x**2, x)
+
+    assert poly_subresultants(f, g) == \
+        (Poly(25 + 225*t + 675*t**2 + 675*t**3, x), [Poly(-x**3 + 5, x),
+                                                     Poly((1 + 3*t)*x**2, x),
+                                                     Poly(5 + 30*t + 45*t**2, x)])
 
 def test_poly_groebner():
     assert poly_groebner(0, x) == [Poly((), x)]
@@ -773,10 +806,10 @@ def test_squarefree():
     A = Poly(x**4 - 3*x**2 + 6, x)
     D = Poly(x**6 - 5*x**4 + 5*x**2 + 4, x)
 
-    f, g = D, A - D.diff(x)*t
+    f, g = D, A - D.diff(x).mul_term(t)
 
-    R = poly_subresultants(f, g)
-    S = poly_sqf(Poly(R[-1], t))
+    res, R = poly_subresultants(f, g)
+    S = poly_sqf(Poly(res, t))
 
     assert S == [Poly(45796, t), Poly(1, t), Poly(4*t**2 + 1, t)]
 
@@ -994,27 +1027,69 @@ def test_roots_quadratic():
         [-1 + I*sqrt(2)/2, -1 - I*sqrt(2)/2]
 
 def test_roots_cubic():
-    assert roots_cubic(Poly(2*x**3, x)) == [0, 0, 0]
-    assert roots_cubic(Poly(x**3-3*x**2+3*x-1, x)) == [1, 1, 1]
-
-    assert roots_cubic(Poly(x**3+1, x)) == \
-        [-1, S.Half - I*sqrt(3)/2, S.Half + I*sqrt(3)/2]
+    # normalized cubic,  x**3+a*x**2+b*x+c, is changed to y**3+p*y+q
+    # check c==0
+    assert roots_cubic(Poly(x**3+2*x**2+3*x+0, x)) == [-1 + I*2**S.Half, 0, -1 - I*2**S.Half]
+    # p==0 and q == 0
+    assert roots_cubic(Poly((x+2)**3,x)) == [-2, -2, -2]
+    # p==0 and q!=0
+    assert set(roots_cubic(Poly(x**3+1, x))) == \
+           set([-S.One, S.Half - I*sqrt(3)/2, S.Half + I*sqrt(3)/2])
+    # p!=0 and q==0
+    assert set(roots_cubic(Poly(x**3 + 3*x**2 + 4*x + 2, x))) == \
+           set([-S.One - I, -S.One, -S.One + I])
+    # p!=0 and q!=0
+    p = Poly(x**3 - 6*x**2 + 13*x - 8, x)
+    ans = roots_cubic(p)
+    assert [[str(round(tmp.evalf().as_real_imag()[j],2)) for tmp in ans]
+            for j in [0,1]] == [['1.0', '2.5', '2.5'],
+                                ['0.0', '-1.32', '1.32']]
 
 def test_roots_quartic():
-    assert roots_quartic(Poly(x**4, x)) == [0, 0, 0, 0]
-    assert roots_quartic(Poly(x**4 + x**3, x)) in [
-        [-1,0,0,0],
-        [0,-1,0,0],
-        [0,0,-1,0],
-        [0,0,0,-1]
-    ]
-    assert roots_quartic(Poly(x**4 - x**3, x)) in [
-        [1,0,0,0],
-        [0,1,0,0],
-        [0,0,1,0],
-        [0,0,0,1]
-    ]
-    assert roots_quartic(Poly(x**4 + x, x)) == [S.Half + I*sqrt(3)/2, S.Half - I*sqrt(3)/2, 0, -1]
+    assert sorted(roots_quartic(Poly(x**4 + x**3, x))) == [-1, 0, 0, 0]
+    assert sorted(roots_quartic(Poly(x**4 - x**3, x))) == [0, 0, 0, 1]
+    assert roots_quartic(Poly(x**4, x)) == [0]*4
+    # x**4+a*x**3+b*x**2+c*x+d=0 reduces to y^4+e*y^2+f*y+g when
+    # replacing x with y-a/4. Check different types of solutions:
+    # check for homogeneous in x (d=0)
+    set(roots_quartic(Poly(x**4 - 6*x**3 + 11*x**2 - 6*x, x))) == \
+                            set([0, 1, 2, 3])
+    # check quasisymmetric (d=(c/a)**2)
+    set(roots_quartic(Poly(x**4 +4*x**3 + 7*x**2 + 8*x + 4, x))) == \
+          set([-1, -2, -S.Half - I*7**S.Half/2, -1/2 + I*7**S.Half/2])
+    # check for homogeneous in y (g=0)
+    ans = roots_quartic(Poly(x**4 + x**3 - Rational(445, 16)*x**2 + x + 2, x))
+    assert set([str(tmp.evalf(2)) for tmp in ans]) == \
+           set(['-2.5e-1', '-5.8 + .0e-7*I', '2.9e-1 + .0e-7*I', '4.8 + .0e-7*I'])
+    # no real roots and f=0
+    assert roots_quartic(Poly(10 + 14*x + 11*x**2 + 4*x**3 + x**4, x)) == \
+           [-1 - I, -1 - 2*I, -1 + I, -1 + 2*I]
+    # no real roots, but f!=0
+    p = Poly(10 + 14*x + 11*x**2 + 4*x**3 + x**4, x)
+    assert set(roots_quartic(p)) == \
+           set([-1 - 2*I, -1 + 2*I, -1 - I, -1 + I])
+    # 2 real roots
+    p = Poly(x**4 - 9*x**3 + 30*x**2 - 42*x + 20, x)
+    ans = roots_quartic(p)
+    assert [[str(tmp.evalf(2).as_real_imag()[j]) for tmp in ans] for j in [0,1]] == \
+           [['3.0', '3.0', '1.0', '2.0'], ['-1.0', '1', '0', '0']]
+    #issue 1468c
+    p = Poly(-1 + 10*x - 32*x**2 + 54*x**3 - 27*x**4, x)
+    ans = roots_quartic(p)
+    assert [[str(tmp.evalf(2).as_real_imag()[j]) for tmp in ans] for j in [0,1]] == \
+           [['2.8e-1', '2.8e-1', '1.6e-1', '1.3'], ['-3.1e-1', '3.1e-1', '0', '0']]
+    # -- a u=0 case
+    i = I*sqrt(41847)
+    p = Poly(x**4 + x**3 + x**2 + (S(3)/8 + i/72)*x + S(1)/96 + i/288, x)
+    ans = roots_quartic(p)
+    assert [[str(tmp.evalf(2).as_real_imag()[j]) for tmp in ans] for j in [0,1]] == \
+    [['-2.5e-1', '-2.5e-1', '8.5e-1', '-1.3'],
+     ['1.6', '-1.1e-2', '-7.8e-1', '-7.8e-1']]
+    # 4 real roots
+    p = Poly(x**4 + -11*x**3 + 41*x**2 - 61*x + 30, x)
+    ans = roots_quartic(p)
+    assert [[str(tmp.evalf(2).as_real_imag()[j]) for tmp in ans] for j in [0,1]] == \
+           [['1.0', '2.0', '3.0', '5.0'], ['6.0e-8', '6.0e-8', '6.0e-8', '6.0e-8']]
 
 def test_roots_binomial():
     assert roots_binomial(Poly(5*x, x)) == [0]
@@ -1032,8 +1107,8 @@ def test_roots_binomial():
     r0 = roots_quadratic(Poly(a1*x**2 + b1, x))
     r1 = roots_binomial(Poly(a1*x**2 + b1, x))
 
-    assert powsimp(r0[0]) == powsimp(r1[0])
-    assert powsimp(r0[1]) == powsimp(r1[1])
+    assert separate(r0[0]) == separate(r1[0])
+    assert separate(r0[1]) == separate(r1[1])
 
 def test_roots_rational():
     assert roots_rational(Poly(x**2-1, x)) == [S.One, -S.One]
@@ -1046,11 +1121,25 @@ def test_roots_rational():
 
 def test_roots():
     assert roots(1, x) == {}
+    assert roots([1]) == {}
+    assert roots(x) == {S.Zero: 1}
     assert roots(x, x) == {S.Zero: 1}
+    uv = Poly(x, x)
+    mv = Poly(x + y, x, y)
+    assert roots(uv) == {S.Zero: 1}
+    assert roots(uv, x) == {S.Zero: 1}
+    assert roots(mv, x) == {-y: 1}
+    raises(SymbolsError,"roots([1],x)")
+    raises(SymbolsError,"roots(x+y, x, y)")
+    raises(SymbolsError,"roots(uv, x, y)")
+    raises(SymbolsError,"roots(mv, x, y)")
+    raises(MultivariatePolyError,"roots(mv)")
+
     assert roots(x**9, x) == {S.Zero: 9}
     assert roots(((x-2)*(x+3)*(x-4)).expand(), x) == {-S(3): 1, S(2): 1, S(4): 1}
 
-    assert roots(2*x+1, x) == {-S.Half: 1}
+    assert roots(2*x+1, x) == roots([2, 1])
+    assert roots([2, 1]) == {-S.Half: 1}
     assert roots((2*x+1)**2, x) == {-S.Half: 2}
     assert roots((2*x+1)**5, x) == {-S.Half: 5}
     assert roots((2*x+1)**10, x) == {-S.Half: 10}
@@ -1103,52 +1192,42 @@ def test_roots():
     r1_2, r1_3, r1_9, r4_9, r19_27 = [ Rational(*r) \
         for r in ((1,2), (1,3), (1,9), (4,9), (19,27)) ]
 
-    assert roots(x**3+x**2-x+1, x, cubics=True) in [
-            {
+    assert set(tmp.evalf() for tmp in roots(x**3+x**2-x+1, x, cubics=True)) == \
+           set(tmp.evalf() for tmp in [
         -r1_3 - (r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
-        r4_9*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3): 1,
+        r4_9*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3),
 
         -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
         r4_9/(r1_2 + r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) + \
-        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3,
 
         -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
         r4_9/(r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) - \
-        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
-            },
-            {
-        -r1_3 - (r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
-        r4_9*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3): 1,
-
-        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 - \
-        r4_9/(-r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) + \
-        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
-
-        -r1_3 + r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3 + \
-        r4_9/(r1_2 - r1_2*I*3**r1_2)*(r19_27 + r1_9*3**r1_2*11**r1_2)**(-r1_3) - \
-        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3: 1,
-            },
-            ]
+        r1_2*I*3**r1_2*(r19_27 + r1_9*3**r1_2*11**r1_2)**r1_3
+            ])
 
     f = (x**2+2*x+3).subs(x, 2*x**2 + 3*x).subs(x, 5*x-4)
 
     r1_2, r13_20, r1_100 = [ Rational(*r) \
         for r in ((1,2), (13,20), (1,100)) ]
-
-    assert roots(f, x) == {
+    ans = {
         r13_20 + r1_100*(25 - 200*I*2**r1_2)**r1_2: 1,
         r13_20 - r1_100*(25 - 200*I*2**r1_2)**r1_2: 1,
         r13_20 + r1_100*(25 + 200*I*2**r1_2)**r1_2: 1,
         r13_20 - r1_100*(25 + 200*I*2**r1_2)**r1_2: 1,
     }
 
-    p = Poly(z**3 + (-2 - y)*z**2 + (1 + 2*y - 2*x**2)*z - y + 2*x**2, z)
+    assert set(tmp.evalf() for tmp in roots(f, x).keys()) == \
+           set(tmp.evalf() for tmp in ans.keys())
 
-    assert roots(p) == {
+    p = Poly(z**3 + (-2 - y)*z**2 + (1 + 2*y - 2*x**2)*z - y + 2*x**2, z)
+    ans = {
         S.One: 1,
         S.Half + S.Half*y + S.Half*(1 - 2*y + y**2 + 8*x**2)**S.Half: 1,
         S.Half + S.Half*y - S.Half*(1 - 2*y + y**2 + 8*x**2)**S.Half: 1,
-    }
+    }.keys()
+
+    assert all(tmp.expand() in ans for tmp in roots(p).keys())
 
     assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x, cubics=False) == {}
     assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x, cubics=True) != {}
@@ -1275,3 +1354,51 @@ def test_factor():
         (1, [(Poly(-y**4*z + x**4, x, y, z), 1),
              (Poly(y**16*z**4 + x**4*y**12*z**3 + x**8*y**8*z**2 + x**12*y**4*z + x**16, x, y, z) , 1)])
 
+def test_discriminant():
+    e = Symbol('e')
+    assert poly_discriminant(Poly(a, x), x) == S(0)
+    assert poly_discriminant(Poly(a*x + b, x), x) == S(1)
+    assert poly_discriminant(Poly(a*x**2 + b*x + c, x), x) == -4*a*c + b**2
+    assert poly_discriminant(Poly(a*x**2 + b*x + c, x), y) == S.Zero
+    assert poly_discriminant(Poly(a*x**3 + b*x**2 + c*x + d, x), x) == \
+    18*a*b*c*d + b**2*c**2 - 27*a**2*d**2 - 4*a*c**3 - 4*d*b**3
+    assert poly_discriminant(Poly(a*x**4 + b*x**3 + c*x**2 + d*x + e, x), x) == \
+    -27*b**4*e**2 + b**2*c**2*d**2 - 128*a**2*c**2*e**2 - 4*a*c**3*d**2 - \
+    192*b*d*a**2*e**2 - 6*a*e*b**2*d**2 + 144*a*c*b**2*e**2 + 144*c*e*a**2*d**2 \
+    - 80*a*b*d*e*c**2 - 4*b**3*d**3 + 256*a**3*e**3 - 4*e*b**2*c**3 + \
+    18*a*b*c*d**3 + 18*c*d*e*b**3 - 27*a**2*d**4 + 16*a*e*c**4
+    assert poly_discriminant(Poly(5*x**5 + x**3 + 2, x), x) == S(31252160)
+    assert poly_discriminant(5*x**5 + x**3 + 2, x) == S(31252160)
+    assert poly_discriminant(Poly(12*x**7 + 15*x**4 + 30*x**3 + x**2 + 1, x), \
+    x) == S(-220289699947514112)
+    # (x - 1)**2*(x + 2 - 3*I)*(x + 2 + 3*I)
+    assert poly_discriminant(Poly(13 - 22*x + 6*x**2 + 2*x**3 + x**4, x), x) == S(0)
+    # (x - 1)*(x + 2 - 3*I)*(x + 2 + 3*I)
+    assert poly_discriminant(Poly(-13 + 9*x + 3*x**2 + x**3, x), x) == S(-11664)
+    assert poly_discriminant(Poly(x**2*y + 2*y, x, y), x) == -8*y**2
+    assert poly_discriminant(Poly(x**2*y + 2*y, x, y), y) == 1
+
+def test_discriminant_5th():
+    skip('takes too much time')
+    # but the result is correct nonetheless
+    e, f = symbols('ef')
+    assert poly_discriminant(Poly(a*x**5 + b*x**4 + c*x**3 + d*x**2 + e*x + f, x), x) \
+    == -128*b**4*d**2*f**2 - 27*a**2*d**4*e**2 - 27*b**2*c**4*f**2 - \
+    4*b**3*d**3*e**2 + 108*a*c**5*f**2 + b**2*c**2*d**2*e**2 - \
+    900*b*a**2*d**3*f**2 - 900*e*a**2*c**3*f**2 - 192*c*e*b**4*f**2 - \
+    50*a**2*b**2*e**2*f**2 - 6*f*b**3*c**2*e**2 - 4*a*c**3*d**2*e**2 + \
+    144*d*f*b**4*e**2 + 144*d*b**3*c**2*f**2 + 825*a**2*c**2*d**2*f**2 + \
+    2000*c*a**3*e**2*f**2 + 2250*e*a**3*d**2*f**2 - 630*a*b*d*c**3*f**2 - \
+    80*c*e*f*b**3*d**2 + 18*a*b*c*d**3*e**2 + 24*a*b*f*c**3*e**2 + \
+    160*a*d*e*b**3*f**2 + 560*a*c*b**2*d**2*f**2 + 560*d*f*a**2*c**2*e**2 + \
+    1020*a*e*b**2*c**2*f**2 + 1020*b*f*a**2*d**2*e**2 - 2050*b*c*d*e*a**2*f**2\
+    - 746*a*c*d*f*b**2*e**2 + 356*a*b*e*f*c**2*d**2 + 256*b**5*f**3 - \
+    4*b**2*c**3*e**3 + 16*a*c**4*e**3 - 3750*c*d*a**3*f**3 - 2500*b*e*a**3*f**3\
+    - 1600*a*c*b**3*f**3 - 1600*d*f*a**3*e**3 - 36*a*f*b**3*e**3 - \
+    6*a*b**2*d**2*e**3 - 4*f*b**2*c**2*d**3 + 16*a*f*c**3*d**3 + \
+    18*c*d*b**3*e**3 + 144*c*a**2*d**2*e**3 + 2000*d*a**2*b**2*f**3 + \
+    2250*b*a**2*c**2*f**3 - 630*c*e*f*a**2*d**3 - 80*a*b*d*c**2*e**3 + \
+    18*d*e*f*b**2*c**3 + 24*a*e*f*b**2*d**3 + 160*b*c*f*a**2*e**3 - \
+    27*b**4*e**4 + 3125*a**4*f**4 - 128*a**2*c**2*e**4 + 16*f*b**3*d**4 - \
+    192*b*d*a**2*e**4 + 144*a*c*b**2*e**4 - 72*a*b*c*f*d**4 - 72*a*d*e*f*c**4 + \
+    256*a**3*e**5 + 108*f*a**2*d**5
