@@ -2,9 +2,15 @@
 A Printer which converts an expression into its LaTeX equivalent.
 """
 
-from sympy.core import S, C, Basic, Symbol
+from sympy.core import S, C, Basic, Add, Symbol, Wild, var
 from printer import Printer
+from conventions import split_super_sub
 from sympy.simplify import fraction
+from sympy import Interval
+
+import sympy.mpmath.libmpf as mlib
+from sympy.mpmath.settings import prec_to_dps
+
 import re
 
 class LatexPrinter(Printer):
@@ -13,22 +19,47 @@ class LatexPrinter(Printer):
     def __init__(self, profile=None):
         Printer.__init__(self)
 
+        mul_symbol_table = {
+            None : r" ",
+            "ldot" : r" \,.\, ",
+            "dot" : r" \cdot ",
+            "times" : r" \times "
+        }
+
         self._settings = {
             "inline" : True,
+            "itex" : False,
+            "descending" : False,
+            "mainvar" : None,
             "fold_frac_powers" : False,
             "fold_func_brackets" : False,
             "mul_symbol" : None,
-            "inv_trig_style" : "abbreviated"
+            "inv_trig_style" : "abbreviated",
+            "mat_str" : "smallmatrix",
+            "mat_delim" : "(",
         }
 
+        self._delim_dict = {'(':')','[':']'}
+
         if profile is not None:
+            if profile.has_key('inline'):
+                if not profile['inline']:
+                    #change to "good" defaults for inline=False before
+                    #updating with the settings from profile
+                    self._settings['mat_str'] = 'bmatrix'
+                    self._settings['mat_delim'] = None
             self._settings.update(profile)
 
+        self._settings['mul_symbol_latex'] = \
+            mul_symbol_table[self._settings['mul_symbol']]
+
     def doprint(self, expr):
         tex = Printer.doprint(self, expr)
 
         if self._settings['inline']:
             return r"$%s$" % tex
+        elif self._settings['itex']:
+            return r"$$%s$$" % tex
         else:
             return r"\begin{equation*}%s\end{equation*}" % tex
 
@@ -45,7 +76,7 @@ class LatexPrinter(Printer):
         Returns True if the expression needs to be wrapped in brackets when
         passed as an argument to a function, False otherwise. This is a more
         liberal version of _needs_brackets, in that many expressions which need
-        to be wrapped in brackets when added/substracted/raised to a power do
+        to be wrapped in brackets when added/subtracted/raised to a power do
         not need them when passed to a function. Such an example is a*b.
         """
         if not self._needs_brackets(expr):
@@ -80,7 +111,37 @@ class LatexPrinter(Printer):
 
     def _print_Add(self, expr):
         args = list(expr.args)
-        args.sort(Basic._compare_pretty)
+
+        if self._settings['mainvar'] is not None:
+            mainvar = self._settings['mainvar']
+            if type(mainvar) == str:
+                mainvar = var(mainvar)
+
+            def compare_exponents(a, b):
+                p1, p2 = Wild("p1"), Wild("p2")
+                r_a = a.match(p1 * mainvar**p2)
+                r_b = b.match(p1 * mainvar**p2)
+                if r_a is None and r_b is None:
+                    c = Basic._compare_pretty(a,b)
+                    return c
+                elif r_a is not None:
+                    if r_b is None:
+                        return 1
+                    else:
+                        c = Basic.compare(r_a[p2], r_b[p2])
+                        if c!=0:
+                            return c
+                        else:
+                            c = Basic._compare_pretty(a,b)
+                            return c
+                elif r_b is not None and r_a is None:
+                    return -1
+
+            args.sort(compare_exponents)
+        else:
+            args.sort(Basic._compare_pretty)
+        if self._settings['descending']:
+            args.reverse()
 
         tex = str(self._print(args[0]))
 
@@ -94,6 +155,31 @@ class LatexPrinter(Printer):
 
         return tex
 
+    def _print_Real(self, expr):
+        # Based off of that in StrPrinter
+        dps = prec_to_dps(expr._prec)
+        str_real = mlib.to_str(expr._mpf_, dps, strip_zeros=True)
+
+        # Must always have a mul symbol (as 2.5 10^{20} just looks odd)
+        seperator = r" \times "
+
+        if self._settings['mul_symbol'] is not None:
+            seperator = self._settings['mul_symbol_latex']
+
+        if 'e' in str_real:
+            (mant, exp) = str_real.split('e')
+
+            if exp[0] == '+':
+                exp = exp[1:]
+
+            return r"%s%s10^{%s}" % (mant, seperator, exp)
+        elif str_real == "+inf":
+            return r"\infty"
+        elif str_real == "-inf":
+            return r"- \intfy"
+        else:
+            return str_real
+
     def _print_Mul(self, expr):
         coeff, terms = expr.as_coeff_terms()
 
@@ -104,40 +190,53 @@ class LatexPrinter(Printer):
             tex = "- "
 
         numer, denom = fraction(C.Mul(*terms))
-
-        mul_symbol_table = {
-            None : r" ",
-            "ldot" : r" \,.\, ",
-            "dot" : r" \cdot ",
-            "times" : r" \times "
-        }
-
-        seperator = mul_symbol_table[self._settings['mul_symbol']]
+        seperator = self._settings['mul_symbol_latex']
 
         def convert(terms):
-            product = []
-
             if not terms.is_Mul:
                 return str(self._print(terms))
             else:
+                _tex = last_term_tex = ""
                 for term in terms.args:
                     pretty = self._print(term)
 
                     if term.is_Add:
-                        product.append(r"\left(%s\right)" % pretty)
+                        term_tex = (r"\left(%s\right)" % pretty)
                     else:
-                        product.append(str(pretty))
+                        term_tex = str(pretty)
 
-                return seperator.join(product)
+                    # between two digits, \times must always be used,
+                    # to avoid confusion
+                    if seperator == " " and \
+                            re.search("[0-9][} ]*$", last_term_tex) and \
+                            re.match("[{ ]*[-+0-9]", term_tex):
+                        _tex += r" \times "
+                    elif _tex:
+                        _tex += seperator
+
+                    _tex += term_tex
+                    last_term_tex = term_tex
+                return _tex
 
         if denom is S.One:
-            if coeff is not S.One:
-                tex += str(self._print(coeff)) + seperator
-
             if numer.is_Add:
-                tex += r"\left(%s\right)" % convert(numer)
+                _tex = r"\left(%s\right)" % convert(numer)
             else:
-                tex += r"%s" % convert(numer)
+                _tex = r"%s" % convert(numer)
+
+            if coeff is not S.One:
+                tex += str(self._print(coeff))
+
+                # between two digits, \times must always be used, to avoid
+                # confusion
+                if seperator == " " and re.search("[0-9][} ]*$", tex) and \
+                        re.match("[{ ]*[-+0-9]", _tex):
+                    tex +=  r" \times " + _tex
+                else:
+                    tex += seperator + _tex
+            else:
+                tex += _tex
+
         else:
             if numer is S.One:
                 if coeff.is_Integer:
@@ -161,13 +260,19 @@ class LatexPrinter(Printer):
         return tex
 
     def _print_Pow(self, expr):
-        if expr.exp.is_Rational and expr.exp.q == 2:
-            base, exp = self._print(expr.base), abs(expr.exp.p)
+        # Treat x**(Rational(1,n)) as special case
+        if expr.exp.is_Rational\
+           and abs(expr.exp.p) == 1\
+           and expr.exp.q != 1:
+            base = self._print(expr.base)
+            expq = expr.exp.q
 
-            if exp == 1:
+            if expq == 2:
                 tex = r"\sqrt{%s}" % base
+            elif self._settings['itex']:
+                tex = r"\root{%d}{%s}" % (expq,base)
             else:
-                tex = r"\sqrt[%s]{%s}" % (exp, base)
+                tex = r"\sqrt[%d]{%s}" % (expq,base)
 
             if expr.exp.is_negative:
                 return r"\frac{1}{%s}" % tex
@@ -200,6 +305,25 @@ class LatexPrinter(Printer):
                 return tex % (self._print(expr.base),
                               self._print(expr.exp))
 
+    def _print_Sum(self, expr):
+        if len(expr.limits) == 1:
+            tex = r"\sum_{%s=%s}^{%s} " % \
+                tuple([ self._print(i) for i in expr.limits[0] ])
+        else:
+            def _format_ineq(l):
+                return r"%s \leq %s \leq %s" % \
+                    tuple([self._print(s) for s in l[1], l[0], l[2]])
+
+            tex = r"\sum_{\substack{%s}} " % \
+                str.join('\\\\', [ _format_ineq(l) for l in expr.limits ])
+
+        if isinstance(expr.function, Add):
+            tex += r"\left(%s\right)" % self._print(expr.function)
+        else:
+            tex += self._print(expr.function)
+
+        return tex
+
     def _print_Derivative(self, expr):
         dim = len(expr.symbols)
 
@@ -239,7 +363,8 @@ class LatexPrinter(Printer):
             tex += r"\int"
 
             if limits is not None:
-                if not self._settings['inline']:
+                if not self._settings['inline'] \
+                   and not self._settings['itex']:
                     tex += r"\limits"
 
                 tex += "_{%s}^{%s}" % (self._print(limits[0]),
@@ -251,13 +376,15 @@ class LatexPrinter(Printer):
             str(self._print(expr.function)), " ".join(symbols))
 
     def _print_Limit(self, expr):
-        tex = r"\lim_{%s \to %s}" % (self._print(expr.var),
-                                     self._print(expr.varlim))
+        e, z, z0, dir = expr.args
 
-        if isinstance(expr.expr, C.AssocOp):
-            return r"%s\left(%s\right)" % (tex, self._print(expr.expr))
+        tex = r"\lim_{%s \to %s}" % (self._print(z),
+                                     self._print(z0))
+
+        if isinstance(e, C.AssocOp):
+            return r"%s\left(%s\right)" % (tex, self._print(e))
         else:
-            return r"%s %s" % (tex, self._print(expr.expr))
+            return r"%s %s" % (tex, self._print(e))
 
     def _print_Function(self, expr, exp=None):
         func = expr.func.__name__
@@ -309,6 +436,9 @@ class LatexPrinter(Printer):
 
             return name % ",".join(args)
 
+    def _print_Poly(self, expr):
+        return self._print(expr.as_basic())
+
     def _print_floor(self, expr, exp=None):
         tex = r"\lfloor{%s}\rfloor" % self._print(expr.args[0])
 
@@ -442,25 +572,9 @@ class LatexPrinter(Printer):
             self._print(expr.args[0])
 
     def _print_Symbol(self, expr):
-        if len(expr.name) == 1:
-            return expr.name
-
-        #convert trailing digits to subscript
-        m = re.match('(^[a-zA-Z]+)([0-9]+)$', expr.name)
-        if m is not None:
-            name, sub=m.groups()
-            tex=self._print_Symbol(Symbol(name))
-            tex="%s_{%s}" %(tex, sub)
-            return tex
-
-        # insert braces to expresions containing '_' or '^'
-        m = re.match('(^[a-zA-Z0-9]+)([_\^]{1})([a-zA-Z0-9]+)$', expr.name)
-        if m is not None:
-            name, sep, rest=m.groups()
-            tex=self._print_Symbol(Symbol(name))
-            tex="%s%s{%s}" %(tex, sep, rest)
-            return tex
+        name, supers, subs = split_super_sub(expr.name)
 
+        # translate name, supers and subs to tex keywords
         greek = set([ 'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta',
                       'eta', 'theta', 'iota', 'kappa', 'lambda', 'mu', 'nu',
                       'xi', 'omicron', 'pi', 'rho', 'sigma', 'tau', 'upsilon',
@@ -469,17 +583,34 @@ class LatexPrinter(Printer):
         other = set( ['aleph', 'beth', 'daleth', 'gimel', 'ell', 'eth',
                       'hbar', 'hslash', 'mho' ])
 
-        if expr.name.lower() in greek:
-            return "\\" + expr.name
-        elif expr.name in other:
-            return "\\" + expr.name
-        else:
-            return expr.name
+        def translate(s):
+            tmp = s.lower()
+            if tmp in greek or tmp in other:
+                return "\\" + s
+            else:
+                return s
+
+        name = translate(name)
+        supers = [translate(sup) for sup in supers]
+        subs = [translate(sub) for sub in subs]
+
+        # glue all items together:
+        if len(supers) > 0:
+            name += "^{%s}" % " ".join(supers)
+        if len(subs) > 0:
+            name += "_{%s}" % " ".join(subs)
+
+        return name
 
     def _print_Relational(self, expr):
+        if self._settings['itex']:
+            lt = r"\lt"
+        else:
+            lt = "<"
+
         charmap = {
             "==" : "=",
-            "<"  : "<",
+            "<"  : lt,
             "<=" : r"\leq",
             "!=" : r"\neq",
         }
@@ -488,30 +619,32 @@ class LatexPrinter(Printer):
             charmap[expr.rel_op], self._print(expr.rhs))
 
     def _print_Piecewise(self, expr):
-        ecpairs = [r"%s & for %s" % (self._print(e), self._print(c)) \
+        ecpairs = [r"%s & \text{for}\: %s" % (self._print(e), self._print(c)) \
                        for e, c in expr.args[:-1]]
-        if expr.args[-1].cond is S.One:
-            ecpairs.append(r"%s & \textrm{otherwise}" % \
+        if expr.args[-1].cond == True:
+            ecpairs.append(r"%s & \text{otherwise}" % \
                                self._print(expr.args[-1].expr))
         else:
-            ecpairs.append(r"%s & for %s" % \
+            ecpairs.append(r"%s & \text{for}\: %s" % \
                            (self._print(expr.args[-1].cond),
                             self._print(expr.args[-1].expr)))
-        tex = r"\left\{\begin{array}{cl} %s \end{array}\right."
+        tex = r"\begin{cases} %s \end{cases}"
         return tex % r" \\".join(ecpairs)
 
     def _print_Matrix(self, expr):
         lines = []
 
-        for line in range(expr.lines): # horrible, should be 'rows'
+        for line in range(expr.rows): # horrible, should be 'rows'
             lines.append(" & ".join([ self._print(i) for i in expr[line,:] ]))
 
-        if self._settings['inline']:
-            tex = r"\left(\begin{smallmatrix}%s\end{smallmatrix}\right)"
-        else:
-            tex = r"\begin{pmatrix}%s\end{pmatrix}"
-
-        return tex % r"\\".join(lines)
+        out_str = r'\begin{%MATSTR%}%s\end{%MATSTR%}'
+        out_str = out_str.replace('%MATSTR%', self._settings['mat_str'])
+        if self._settings['mat_delim']:
+            left_delim = self._settings['mat_delim']
+            right_delim = self._delim_dict[left_delim]
+            out_str = r'\left' + left_delim + out_str + \
+                      r'\right' + right_delim
+        return out_str % r"\\".join(lines)
 
     def _print_tuple(self, expr):
         return r"\begin{pmatrix}%s\end{pmatrix}" % \
@@ -540,25 +673,72 @@ class LatexPrinter(Printer):
             self._print(expr.args[1]), self._print(expr.args[0]))
         return tex
 
+    def _print_Interval(self, i):
+        if i.start == i.end:
+            return r"\left{%s\right}" % self._print(i.start)
+
+        else:
+            if i.left_open:
+                left = '('
+            else:
+                left = '['
+
+            if i.right_open:
+                right = ')'
+            else:
+                right = ']'
+
+            return r"\left%s%s, %s\right%s" % \
+                   (left, self._print(i.start), self._print(i.end), right)
+
+    def _print_Union(self, u):
+        other_sets, singletons = [], []
+        for set in u.args:
+            if isinstance(set, Interval) and set.measure == 0:
+                singletons.append(set.start)
+            else:
+                other_sets.append(set)
+
+        S2 = r"%s" % \
+             r" \cup ".join([ self._print_Interval(i) for i in other_sets ])
+
+        if len(singletons) > 0:
+            S1 = r"\left\{%s\right\}" % \
+                 r", ".join([ self._print(i) for i in singletons ])
+
+            S = r"%s \cup %s" % (S1, S2)
+        else:
+            S = S2
+
+        return S
+
+    def _print_EmptySet(self, e):
+        return r"\emptyset"
+
 def latex(expr, profile=None, **kargs):
     r"""Convert the given expression to LaTeX representation.
 
         You can specify how the generated code will be delimited.
         If the 'inline' keyword is set then inline LaTeX $ $ will
         be used. Otherwise the resulting code will be enclosed in
-        'equation*' environment (remember to import 'amsmath').
+        'equation*' environment (remember to import 'amsmath'),
+        unless the 'itex' keyword is set in which case the itex
+        $$ $$ syntax will be used.
 
-        >>> from sympy import *
-        >>> from sympy.abc import *
+        >>> from sympy import latex, Rational
+        >>> from sympy.abc import x, y, mu, tau
 
         >>> latex((2*tau)**Rational(7,2))
-        '$8 \\sqrt{2} \\sqrt[7]{\\tau}$'
+        '$8 \\sqrt{2} \\tau^{\\frac{7}{2}}$'
 
         >>> latex((2*mu)**Rational(7,2), inline=False)
-        '\\begin{equation*}8 \\sqrt{2} \\sqrt[7]{\\mu}\\end{equation*}'
+        '\\begin{equation*}8 \\sqrt{2} \\mu^{\\frac{7}{2}}\\end{equation*}'
+
+        >>> latex((2*mu)**Rational(7,2), inline=False, itex=True)
+        '$$8 \\sqrt{2} \\mu^{\\frac{7}{2}}$$'
 
         Besides all Basic based expressions, you can recursively
-        convert Pyhon containers (lists, tuples and dicts) and
+        convert Python containers (lists, tuples and dicts) and
         also SymPy matrices:
 
         >>> latex([2/x, y])
