@@ -2,6 +2,7 @@ from sympy.core import S, C, Basic
 from sympy.printing.printer import Printer
 from sympy.printing.str import sstr
 from stringpict import prettyForm, stringPict
+from sympy import Interval
 
 from pretty_symbology import xstr, hobj, vobj, xobj, xsym, pretty_symbol,\
         pretty_atom, pretty_use_unicode, pretty_try_use_unicode, greek
@@ -12,7 +13,7 @@ pprint_try_use_unicode = pretty_try_use_unicode
 
 
 class PrettyPrinter(Printer):
-    """Printer, which converts an expression into 2D ascii-art figure."""
+    """Printer, which converts an expression into 2D ASCII-art figure."""
 
     def __init__(self, profile=None):
         Printer.__init__(self)
@@ -21,20 +22,21 @@ class PrettyPrinter(Printer):
         self._settings = {
                 "full_prec" : "auto",
                 "use_unicode" : True,
+                "wrap_line" : True,
         }
 
         if profile is not None:
             self._settings.update(profile)
 
     def doprint(self, expr):
-        return self._print(expr).terminal_string()
+        return self._print(expr).render(**self._settings)
 
     # empty op so _print(stringPict) returns the same
     def _print_stringPict(self, e):
         return e
 
     def _print_basestring(self, e):
-        return prettyForm(repr(e))
+        return prettyForm(e)
 
     def _print_Symbol(self, e):
         symb = pretty_symbol(e.name)
@@ -58,6 +60,7 @@ class PrettyPrinter(Printer):
     # Infinity inherits from Rational, so we have to override _print_XXX order
     _print_Infinity         = _print_Atom
     _print_NegativeInfinity = _print_Atom
+    _print_EmptySet         = _print_Atom
 
     def _print_Factorial(self, e):
         x = e.args[0]
@@ -186,9 +189,16 @@ class PrettyPrinter(Printer):
 
 
             if ab is not None:
-                # Create pretty forms for endpoints, if definite integral
-                prettyA = self._print(ab[0])
-                prettyB = self._print(ab[1])
+                # Create pretty forms for endpoints, if definite integral.
+                # Do not print empty endpoints.
+                if ab[0] is None:
+                    prettyA = prettyForm("")
+                else:
+                    prettyA = self._print(ab[0])
+                if ab[1] is None:
+                    prettyB = prettyForm("")
+                else:
+                    prettyB = self._print(ab[1])
 
                 if ascii_mode:  # XXX hack
                     # Add spacing so that endpoint can more easily be
@@ -250,7 +260,7 @@ class PrettyPrinter(Printer):
     def _print_Matrix(self, e):
         M = e   # matrix
         S = {}  # i,j -> pretty(M[i,j])
-        for i in range(M.lines):
+        for i in range(M.rows):
             for j in range(M.cols):
                 S[i,j] = self._print(M[i,j])
 
@@ -262,13 +272,13 @@ class PrettyPrinter(Printer):
         maxw = [-1] * M.cols
 
         for j in range(M.cols):
-            maxw[j] = max([S[i,j].width()  for i in range(M.lines)])
+            maxw[j] = max([S[i,j].width()  for i in range(M.rows)])
 
 
         # drawing result
         D = None
 
-        for i in range(M.lines):
+        for i in range(M.rows):
 
             D_row = None
             for j in range(M.cols):
@@ -317,7 +327,7 @@ class PrettyPrinter(Printer):
         P = {}
         for n, ec in enumerate(pexpr.args):
             P[n,0] = self._print(ec.expr)
-            if ec.cond is S.One:
+            if ec.cond == True:
                 P[n,1] = prettyForm('otherwise')
             else:
                 P[n,1] = prettyForm(*prettyForm('for ').right(self._print(ec.cond)))
@@ -532,8 +542,46 @@ class PrettyPrinter(Printer):
         else:
             return self.emptyPrinter(r)
 
+    def _print_Interval(self, i):
+        if i.start == i.end:
+            return self._print_seq(i.args[:1], '{', '}')
 
-    def _print_seq(self, seq, left=None, right=None):
+        else:
+            if i.left_open:
+                left = '('
+            else:
+                left = '['
+
+            if i.right_open:
+                right = ')'
+            else:
+                right = ']'
+
+            return self._print_seq(i.args[:2], left, right)
+
+    def _print_Union(self, u):
+        other_sets, singletons = [], []
+        for set in u.args:
+            if isinstance(set, Interval) and set.measure == 0:
+                singletons.append(set.start)
+            else:
+                other_sets.append(set)
+
+        union_delimiter = ' %s ' % pretty_atom('Union')
+
+        S2 = self._print_seq(other_sets, None, None, union_delimiter)
+
+        if len(singletons) > 0:
+            S1 = self._print_seq(singletons, '{', '}')
+
+            S = prettyForm(*stringPict.next(S1, union_delimiter))
+            S = prettyForm(*stringPict.next(S, S2))
+        else:
+            S = S2
+
+        return S
+
+    def _print_seq(self, seq, left=None, right=None, delimiter=', '):
         S = None
 
         for item in seq:
@@ -543,7 +591,7 @@ class PrettyPrinter(Printer):
                 # first element
                 S = pform
             else:
-                S = prettyForm(*stringPict.next(S, ', '))
+                S = prettyForm(*stringPict.next(S, delimiter))
                 S = prettyForm(*stringPict.next(S, pform))
 
         if S is None:
@@ -591,9 +639,15 @@ class PrettyPrinter(Printer):
 
 def pretty(expr, profile=None, **kargs):
     """
-    Returns a string containing the prettified form of expr. If use_unicode
-    is set to True then certain expressions will use unicode characters,
-    such as the greek letter pi for Pi instances.
+    Returns a string containing the prettified form of expr.
+
+    Arguments
+    ---------
+    expr: the expression to print
+    wrap_line: line wrapping enabled/disabled, should be a boolean value (default to True)
+    use_unicode: use unicode characters, such as the Greek letter pi instead of
+        the string pi. Values should be boolean or None
+    full_prec: use full precision. Default to "auto"
     """
     if profile is not None:
         profile.update(kargs)
@@ -612,6 +666,6 @@ def pretty_print(expr, use_unicode=None):
 
     pprint is just a shortcut for this function
     """
-    print pretty(expr, use_unicode)
+    print pretty(expr, use_unicode = use_unicode)
 
 pprint = pretty_print
