@@ -1,3 +1,4 @@
+from sympy.simplify import powsimp
 """
 Limits
 ======
@@ -92,18 +93,9 @@ Debugging
 
 Because the gruntz algorithm is highly recursive, it's difficult to figure out
 what went wrong inside a debugger. Instead, turn on nice debug prints by
-applying the following patch:
+defining the environment variable SYMPY_DEBUG. For example:
 
-@@ -101,7 +108,7 @@ def debug(func):
-     It will print a nice execution tree with arguments and results
-     of all decorated functions.
-     ""
--    if 1:
-+    if 0:
-         #normal mode - do nothing
-         return func
-
-Then start isympy and type the failing limit. For example:
+[user@localhost]: SYMPY_DEBUG=True ./bin/isympy
 
 In [1]: limit(sin(x)/x, x, 0)
 limitinf(_x*sin(1/_x), _x) = 1
@@ -124,7 +116,7 @@ And check manually which line is wrong. Then go to the source code and debug
 this function to figure out the exact problem.
 
 """
-
+from sympy import SYMPY_DEBUG
 from sympy.core import Basic, S, Add, Mul, Pow, Function, oo, Symbol, \
         Rational, Real, sympify, C, I
 from sympy.functions import log, exp
@@ -137,7 +129,7 @@ def debug(func):
     It will print a nice execution tree with arguments and results
     of all decorated functions.
     """
-    if 1:
+    if not SYMPY_DEBUG:
         #normal mode - do nothing
         return func
 
@@ -204,6 +196,7 @@ def compare(a,b,x):
 @debug
 def mrv(e, x):
     "Returns a python set of  most rapidly varying (mrv) subexpressions of 'e'"
+    e = powsimp(e, deep=True, combine='exp')
     assert isinstance(e, Basic)
     if not e.has(x):
         return set([])
@@ -286,9 +279,13 @@ def rewrite(e,Omega,x,wsym):
         O2.append(exp((f.args[0]-c[0]*g.args[0]).expand())*wsym**c[0])
     #Remember that Omega contains subexpressions of "e". So now we find
     #them in "e" and substitute them for our rewriting, stored in O2
-    f=e
-    for a,b in zip(Omega,O2):
-        f=f.subs(a,b)
+    f = e
+    # the following powsimp is necessary to automatically combine exponentials,
+    # so that the .subs() below succeeds:
+    from sympy import powsimp
+    f = powsimp(f, deep=True, combine='exp')
+    for a, b in zip(Omega, O2):
+        f = f.subs(a, b)
 
     #finally compute the logarithm of w (logw).
     logw=g.args[0]
@@ -330,7 +327,7 @@ def sign(e, x):
         return sign(e.args[0] -1, x)
     elif e.is_Add:
         return sign(limitinf(e, x), x)
-    raise "cannot determine the sign of %s"%e
+    raise ValueError("Cannot determine the sign of %s" % e)
 
 @debug
 def limitinf(e, x):
@@ -368,7 +365,7 @@ def subexp(e,sub):
 def calculate_series(e, x):
     """ Calculates at least one term of the series of "e" in "x".
 
-    This is a place that fails most often, so it is in it's own function.
+    This is a place that fails most often, so it is in its own function.
     """
 
     f = e
@@ -425,16 +422,16 @@ class Limit2(Basic):
 
 
     def __pretty__(self):
-         e, x, t = [a.__pretty__() for a in (self.e,self.x,self.x0)]
-         a = prettyForm('lim')
-         a = prettyForm(*a.below('%s->%s' % (x, t)))
-         a = prettyForm(*stringPict.next(a, e))
-         return a
+        e, x, t = [a.__pretty__() for a in (self.e,self.x,self.x0)]
+        a = prettyForm('lim')
+        a = prettyForm(*a.below('%s->%s' % (x, t)))
+        a = prettyForm(*stringPict.next(a, e))
+        return a
 
     def __latex__(self):
-         return r"\lim_{%s \to %s}%s" % (self.x.__latex__(), \
-                                                 self.x0.__latex__(),
-                                                 self.e.__latex__() )
+        return r"\lim_{%s \to %s}%s" % (self.x.__latex__(), \
+                                                self.x0.__latex__(),
+                                                self.e.__latex__() )
 
     @property
     def e(self):
@@ -448,8 +445,13 @@ class Limit2(Basic):
     def x0(self):
         return self._args[2]
 
-    def doit(self):
-        return limit(self.e,self.x,self.x0)
+    def doit(self, **hints):
+        if hints.get('deep', True):
+            e = self.e.doit(**hints)
+            z = self.z.doit(**hints)
+            z0 = self.z0.doit(**hints)
+
+        return limit(e, x, x0)
 
     def __mathml__(self):
         if self._mathml:
