@@ -1,4 +1,4 @@
-from sympy.core import sympify, Lambda, Symbol, Integer, Rational, oo, Real
+from sympy.core import sympify, Lambda, Symbol, Integer, Rational, oo, Real, pi
 from sympy.functions import sqrt, exp, erf
 from sympy.printing import _StrPrinter as StrPrinter
 import random
@@ -9,18 +9,19 @@ class Sample(tuple):
     Sample([x1, x2, x3, ...]) represents a collection of samples.
     Sample parameters like mean, variance and stddev can be accessed as
     properties.
+    The sample will be sorted.
     """
     def __new__(cls, sample):
-        s = tuple.__new__(cls, sample)
+        s = tuple.__new__(cls, sorted(sample))
         s.mean = mean = sum(s) / Integer(len(s))
         s.variance = sum([(x-mean)**2 for x in s]) / Integer(len(s))
         s.stddev = sqrt(s.variance)
+        if len(s) % 2:
+            s.median = s[len(s)//2]
+        else:
+            s.median = sum(s[len(s)//2-1:len(s)//2+1]) / Integer(2)
         return s
 
-    @property
-    def median(self):
-        raise NotImplementedError
-
     def __repr__(self):
         return StrPrinter.doprint(self)
 
@@ -54,6 +55,8 @@ class Normal(ContinuousProbability):
 
     Example usage:
 
+        >>> from sympy.statistics import Normal
+        >>> from sympy import oo
         >>> N = Normal(1, 2)
         >>> N.mean
         1
@@ -66,7 +69,7 @@ class Normal(ContinuousProbability):
         >>> N.probability(-oo, oo)
         1
         >>> N.probability(-1, 3)
-        erf(1/2*2**(1/2))
+        erf(2**(1/2)/2)
         >>> _.evalf()
         0.682689492137086
 
@@ -101,6 +104,7 @@ class Normal(ContinuousProbability):
 
         Examples usage:
             # One standard deviation
+            >>> from sympy.statistics import Normal
             >>> N = Normal(0, 1)
             >>> N.confidence(0.68)
             (-0.994457883209753, 0.994457883209753)
@@ -113,6 +117,7 @@ class Normal(ContinuousProbability):
             (-1.95996398454005, 1.95996398454005)
             >>> N.probability(*_).evalf()
             0.950000000000000
+
         """
 
         if p == 1:
@@ -124,12 +129,10 @@ class Normal(ContinuousProbability):
         # error function is not yet implemented in SymPy but can easily be
         # computed numerically
 
-        from sympy.mpmath import mpf, erf
-        from sympy.mpmath.calculus import secant
+        from sympy.mpmath import mpf, erfinv
 
-        p = mpf(p)
         # calculate y = ierf(p) by solving erf(y) - p = 0
-        y = secant(lambda y: erf(y) - p, 0)
+        y = erfinv(mpf(p))
         t = Real(str(mpf(float(s.sigma)) * mpf(2)**0.5 * y))
         mu = s.mu.evalf()
         return (mu-t, mu+t)
@@ -187,11 +190,14 @@ class Uniform(ContinuousProbability):
     def confidence(s, p):
         """Generate a symmetric (p*100)% confidence interval.
 
+        >>> from sympy import Rational
+        >>> from sympy.statistics import Uniform
         >>> U = Uniform(1, 2)
         >>> U.confidence(1)
         (1, 2)
         >>> U.confidence(Rational(1,2))
         (5/4, 7/4)
+
         """
         p = sympify(p)
         assert p <= 1
@@ -219,19 +225,21 @@ class PDF(ContinuousProbability):
 
     Example usage:
 
-        >>> from sympy import Symbol
-        >>> x = Symbol('x')
+        >>> from sympy import Symbol, exp, oo
+        >>> from sympy.statistics.distributions import PDF
+        >>> from sympy.abc import x
         >>> a = Symbol('a', positive=True)
 
         >>> exponential = PDF(exp(-x/a)/a, (x,0,oo))
         >>> exponential.pdf(x)
-        1/a*exp(-x/a)
+        exp(-x/a)/a
         >>> exponential.cdf(x)
         1 - exp(-x/a)
         >>> exponential.mean
         a
         >>> exponential.variance
         a**2
+
     """
 
     def __init__(self, pdf, var):
@@ -255,13 +263,15 @@ class PDF(ContinuousProbability):
 
         Example usage:
 
-            >>> from sympy import Symbol
-            >>> x = Symbol('x')
+            >>> from sympy import Symbol, exp, oo
+            >>> from sympy.statistics.distributions import PDF
+            >>> from sympy.abc import x
             >>> a = Symbol('a', positive=True)
 
             >>> exponential = PDF(exp(-x/a), (x,0,oo))
             >>> exponential.normalize().pdf(x)
-            1/a*exp(-x/a)
+            exp(-x/a)/a
+
         """
 
         norm = self.probability(*self.domain)
